@@ -9,7 +9,6 @@ module/script to function.
 Note that this does *not* currently automate the process of creating structs
 or dtypes for those structs.  They should be added manually in the template file.
 """
-from __future__ import absolute_import, division, print_function
 # note that we do *not* use unicode_literals here, because that makes the
 # generated code's strings have u'' in them on py 2.x
 
@@ -18,19 +17,19 @@ import os.path
 from collections import OrderedDict
 
 
-ctype_to_dtype = {'double'     : "numpy.double",
-                  'int'        : "numpy.intc",
-                  'eraASTROM'  : "dt_eraASTROM",
-                  'eraLDBODY'  : "dt_eraLDBODY",
-                  'char'       : "numpy.dtype('S1')",
-                  'const char' : "numpy.dtype('S16')",
+ctype_to_dtype = {'double': "numpy.double",
+                  'int': "numpy.intc",
+                  'eraASTROM': "dt_eraASTROM",
+                  'eraLDBODY': "dt_eraLDBODY",
+                  'char': "numpy.dtype('S1')",
+                  'const char': "numpy.dtype('S16')",
                   }
 
 
-NDIMS_REX = re.compile(re.escape("numpy.dtype([('fi0', '.*', <(.*)>)])").replace(r'\.\*','.*').replace(r'\<', '(').replace(r'\>',')'))
+NDIMS_REX = re.compile(re.escape("numpy.dtype([('fi0', '.*', <(.*)>)])").replace(r'\.\*', '.*').replace(r'\<', '(').replace(r'\>', ')'))
 
 
-class FunctionDoc(object):
+class FunctionDoc:
 
     def __init__(self, doc):
         self.doc = doc.replace("**", "  ").replace("/*\n", "").replace("*/", "")
@@ -99,7 +98,7 @@ class FunctionDoc(object):
         return self.doc.replace("  \n", "\n")
 
 
-class ArgumentDoc(object):
+class ArgumentDoc:
 
     def __init__(self, doc):
         match = re.search("^ +([^ ]+)[ ]+([^ ]+)[ ]+(.+)", doc)
@@ -116,7 +115,7 @@ class ArgumentDoc(object):
         return "    {0:15} {1:15} {2}".format(self.name, self.type, self.doc)
 
 
-class Argument(object):
+class Argument:
 
     def __init__(self, definition, doc):
         self.doc = doc
@@ -155,21 +154,21 @@ class Argument(object):
 
     @property
     def ctype_ptr(self):
-        if (self.is_ptr) | (len(self.shape)>0):
+        if (self.is_ptr) | (len(self.shape) > 0):
             return self.ctype+" *"
         else:
             return self.ctype
 
     @property
     def name_in_broadcast(self):
-        if len(self.shape)>0:
+        if len(self.shape) > 0:
             return "{0}_in[...{1}]".format(self.name, ",0"*len(self.shape))
         else:
             return "{0}_in".format(self.name)
 
     @property
     def name_out_broadcast(self):
-        if len(self.shape)>0:
+        if len(self.shape) > 0:
             return "{0}_out[...{1}]".format(self.name, ",0"*len(self.shape))
         else:
             return "{0}_out".format(self.name)
@@ -197,7 +196,7 @@ class Argument(object):
         return "Argument('{0}', name='{1}', ctype='{2}', inout_state='{3}')".format(self.definition, self.name, self.ctype, self.inout_state)
 
 
-class ReturnDoc(object):
+class ReturnDoc:
 
     def __init__(self, doc):
         self.doc = doc
@@ -227,7 +226,7 @@ class ReturnDoc(object):
         return "Return value, type={0:15}, {1}, {2}".format(self.type, self.descr, self.doc)
 
 
-class Return(object):
+class Return:
 
     def __init__(self, ctype, doc):
         self.name = 'c_retval'
@@ -258,7 +257,7 @@ class Return(object):
         return self.doc.ret_info
 
 
-class Function(object):
+class Function:
     """
     A class representing a C function.
 
@@ -300,15 +299,15 @@ class Function(object):
             else:
                 filecontents = f.read()
 
-        pattern = "\n([^\n]+{0} ?\([^)]+\)).+?(/\*.+?\*/)".format(name)
-        p = re.compile(pattern, flags=re.DOTALL|re.MULTILINE)
+        pattern = r"\n([^\n]+{0} ?\([^)]+\)).+?(/\*.+?\*/)".format(name)
+        p = re.compile(pattern, flags=re.DOTALL | re.MULTILINE)
 
         search = p.search(filecontents)
         self.cfunc = " ".join(search.group(1).split())
         self.doc = FunctionDoc(search.group(2))
 
         self.args = []
-        for arg in re.search("\(([^)]+)\)", self.cfunc).group(1).split(', '):
+        for arg in re.search(r"\(([^)]+)\)", self.cfunc).group(1).split(', '):
             self.args.append(Argument(arg, self.doc))
         self.ret = re.search("^(.*){0}".format(name), self.cfunc).group(1).strip()
         if self.ret != 'void':
@@ -347,14 +346,79 @@ class Function(object):
         return "Function(name='{0}', pyname='{1}', filename='{2}', filepath='{3}')".format(self.name, self.pyname, self.filename, self.filepath)
 
 
-class Constant(object):
+class Constant:
 
     def __init__(self, name, value, doc):
-        self.name = name.replace("ERFA_","")
-        self.value = value.replace("ERFA_","")
+        self.name = name.replace("ERFA_", "")
+        self.value = value.replace("ERFA_", "")
         self.doc = doc
 
 
+class ExtraFunction(Function):
+    """
+    An "extra" function - e.g. one not following the SOFA/ERFA standard format.
+
+    Parameters
+    ----------
+    cname : str
+        The name of the function in C
+    prototype : str
+        The prototype for the function (usually derived from the header)
+    pathfordoc : str
+        The path to a file that contains the prototype, with the documentation
+        as a multiline string *before* it.
+    """
+
+    def __init__(self, cname, prototype, pathfordoc):
+        self.name = cname
+        self.pyname = cname.split('era')[-1].lower()
+        self.filepath, self.filename = os.path.split(pathfordoc)
+
+        self.prototype = prototype.strip()
+        if prototype.endswith('{') or prototype.endswith(';'):
+            self.prototype = prototype[:-1].strip()
+
+        incomment = False
+        lastcomment = None
+        with open(pathfordoc, 'r') as f:
+            for l in f:
+                if incomment:
+                    if l.lstrip().startswith('*/'):
+                        incomment = False
+                        lastcomment = ''.join(lastcomment)
+                    else:
+                        if l.startswith('**'):
+                            l = l[2:]
+                        lastcomment.append(l)
+                else:
+                    if l.lstrip().startswith('/*'):
+                        incomment = True
+                        lastcomment = []
+                    if l.startswith(self.prototype):
+                        self.doc = lastcomment
+                        break
+            else:
+                raise ValueError('Did not find prototype {} in file '
+                                 '{}'.format(self.prototype, pathfordoc))
+
+        self.args = []
+        argset = re.search(r"{0}\(([^)]+)?\)".format(self.name),
+                           self.prototype).group(1)
+        if argset is not None:
+            for arg in argset.split(', '):
+                self.args.append(Argument(arg, self.doc))
+        self.ret = re.match("^(.*){0}".format(self.name),
+                            self.prototype).group(1).strip()
+        if self.ret != 'void':
+            self.args.append(Return(self.ret, self.doc))
+
+    def __repr__(self):
+        r = super().__repr__()
+        if r.startswith('Function'):
+            r = 'Extra' + r
+        return r
+
+
 def main(srcdir, outfn, templateloc, verbose=True):
     from jinja2 import Environment, FileSystemLoader
 
@@ -363,13 +427,15 @@ def main(srcdir, outfn, templateloc, verbose=True):
     else:
         print_ = lambda *args, **kwargs: None
 
-    #Prepare the jinja2 templating environment
+    # Prepare the jinja2 templating environment
     env = Environment(loader=FileSystemLoader(templateloc))
 
     def prefix(a_list, pre):
         return [pre+'{0}'.format(an_element) for an_element in a_list]
+
     def postfix(a_list, post):
         return ['{0}'.format(an_element)+post for an_element in a_list]
+
     def surround(a_list, pre, post):
         return [pre+'{0}'.format(an_element)+post for an_element in a_list]
     env.filters['prefix'] = prefix
@@ -379,7 +445,7 @@ def main(srcdir, outfn, templateloc, verbose=True):
     erfa_c_in = env.get_template('core.c.templ')
     erfa_py_in = env.get_template('core.py.templ')
 
-    #Extract all the ERFA function names from erfa.h
+    # Extract all the ERFA function names from erfa.h
     if os.path.isdir(srcdir):
         erfahfn = os.path.join(srcdir, 'erfa.h')
         multifilserc = True
@@ -391,14 +457,14 @@ def main(srcdir, outfn, templateloc, verbose=True):
         erfa_h = f.read()
 
     funcs = OrderedDict()
-    section_subsection_functions = re.findall('/\* (\w*)/(\w*) \*/\n(.*?)\n\n',
-                                              erfa_h, flags=re.DOTALL|re.MULTILINE)
+    section_subsection_functions = re.findall(r'/\* (\w*)/(\w*) \*/\n(.*?)\n\n',
+                                              erfa_h, flags=re.DOTALL | re.MULTILINE)
     for section, subsection, functions in section_subsection_functions:
         print_("{0}.{1}".format(section, subsection))
         if ((section == "Astronomy") or (subsection == "AngleOps")
             or (subsection == "SphericalCartesian")
             or (subsection == "MatrixVectorProducts")):
-            func_names = re.findall(' (\w+)\(.*?\);', functions, flags=re.DOTALL)
+            func_names = re.findall(r' (\w+)\(.*?\);', functions, flags=re.DOTALL)
             for name in func_names:
                 print_("{0}.{1}.{2}...".format(section, subsection, name))
                 if multifilserc:
@@ -409,7 +475,7 @@ def main(srcdir, outfn, templateloc, verbose=True):
                     # the start of the header declaration, otherwise it
                     # might find a *call* of the function instead of the
                     # definition
-                    for line in functions.split('\n'):
+                    for line in functions.split(r'\n'):
                         if name in line:
                             # [:-1] is to remove trailing semicolon, and
                             # splitting on '(' is because the header and
@@ -427,18 +493,29 @@ def main(srcdir, outfn, templateloc, verbose=True):
 
     funcs = list(funcs.values())
 
-    #Extract all the ERFA constants from erfam.h
+    # Extract all the ERFA constants from erfam.h
     erfamhfn = os.path.join(srcdir, 'erfam.h')
     with open(erfamhfn, 'r') as f:
         erfa_m_h = f.read()
     constants = []
     for chunk in erfa_m_h.split("\n\n"):
-        result = re.findall("#define (ERFA_\w+?) (.+?)$", chunk, flags=re.DOTALL|re.MULTILINE)
+        result = re.findall(r"#define (ERFA_\w+?) (.+?)$", chunk, flags=re.DOTALL | re.MULTILINE)
         if result:
-            doc = re.findall("/\* (.+?) \*/\n", chunk, flags=re.DOTALL)
+            doc = re.findall(r"/\* (.+?) \*/\n", chunk, flags=re.DOTALL)
             for (name, value) in result:
                 constants.append(Constant(name, value, doc))
 
+    # TODO: re-enable this when const char* return values and non-status code integer rets are possible
+    # #Add in any "extra" functions from erfaextra.h
+    # erfaextrahfn = os.path.join(srcdir, 'erfaextra.h')
+    # with open(erfaextrahfn, 'r') as f:
+    #     for l in f:
+    #         ls = l.strip()
+    #         match = re.match('.* (era.*)\(', ls)
+    #         if match:
+    #             print_("Extra:  {0} ...".format(match.group(1)))
+    #             funcs.append(ExtraFunction(match.group(1), ls, erfaextrahfn))
+
     print_("Rendering template")
     erfa_c = erfa_c_in.render(funcs=funcs)
     erfa_py = erfa_py_in.render(funcs=funcs, constants=constants)
@@ -455,6 +532,7 @@ def main(srcdir, outfn, templateloc, verbose=True):
 
     return erfa_c, erfa_py, funcs
 
+
 DEFAULT_ERFA_LOC = os.path.join(os.path.split(__file__)[0],
                                 '../../cextern/erfa')
 DEFAULT_TEMPLATE_LOC = os.path.split(__file__)[0]
