@@ -1,16 +1,14 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-import io
 import os
 import sys
 import subprocess
 
+import pytest
+
 from ...tests.helper import catch_warnings
-from ...extern import six
 
 from ...utils.data import get_pkg_data_filename
 from .. import configuration
@@ -120,8 +118,6 @@ def test_configitem():
 
 def test_configitem_types():
 
-    from ...tests.helper import pytest
-
     from ..configuration import ConfigNamespace, ConfigItem
 
     cio = ConfigItem(['op1', 'op2', 'op3'])
@@ -137,7 +133,7 @@ def test_configitem_types():
     assert isinstance(conf.tstnm1, int)
     assert isinstance(conf.tstnm2, float)
     assert isinstance(conf.tstnm3, bool)
-    assert isinstance(conf.tstnm4, six.text_type)
+    assert isinstance(conf.tstnm4, str)
 
     with pytest.raises(TypeError):
         conf.tstnm1 = 34.3
@@ -150,7 +146,6 @@ def test_configitem_types():
 
 def test_configitem_options(tmpdir):
 
-    from ...tests.helper import pytest
     from ..configuration import ConfigNamespace, ConfigItem, get_config
 
     cio = ConfigItem(['op1', 'op2', 'op3'])
@@ -162,7 +157,7 @@ def test_configitem_options(tmpdir):
 
     sec = get_config(cio.module)
 
-    assert isinstance(cio(), six.text_type)
+    assert isinstance(cio(), str)
     assert cio() == 'op1'
     assert sec['tstnmo'] == 'op1'
 
@@ -176,9 +171,9 @@ def test_configitem_options(tmpdir):
     while apycfg.parent is not apycfg:
         apycfg = apycfg.parent
     f = tmpdir.join('astropy.cfg')
-    with io.open(f.strpath, 'wb') as fd:
+    with open(f.strpath, 'wb') as fd:
         apycfg.write(fd)
-    with io.open(f.strpath, 'rU', encoding='utf-8') as fd:
+    with open(f.strpath, 'r', encoding='utf-8') as fd:
         lns = [x.strip() for x in f.readlines()]
 
     assert 'tstnmo = op2' in lns
@@ -189,7 +184,6 @@ def test_config_noastropy_fallback(monkeypatch):
     Tests to make sure configuration items fall back to their defaults when
     there's a problem accessing the astropy directory
     """
-    from ...tests.helper import pytest
 
     # make sure the config directory is not searched
     monkeypatch.setenv(str('XDG_CONFIG_HOME'), 'foo')
@@ -242,7 +236,7 @@ def test_configitem_setters():
     try:
         with conf.set_temp('tstnm12', 47):
             raise Exception
-    except:
+    except Exception:
         pass
 
     assert conf.tstnm12 == 43
@@ -252,7 +246,7 @@ def test_empty_config_file():
     from ..configuration import is_unedited_config_file
 
     def get_content(fn):
-        with io.open(get_pkg_data_filename(fn), 'rt', encoding='latin-1') as fd:
+        with open(get_pkg_data_filename(fn), 'rt', encoding='latin-1') as fd:
             return fd.read()
 
     content = get_content('data/empty.cfg')
@@ -268,7 +262,7 @@ def test_empty_config_file():
     assert is_unedited_config_file(content)
 
 
-class TestAliasRead(object):
+class TestAliasRead:
 
     def setup_class(self):
         configuration._override_config_file = get_pkg_data_filename('data/alias.cfg')
@@ -305,7 +299,7 @@ def test_configitem_unicode(tmpdir):
 
     sec = get_config(cio.module)
 
-    assert isinstance(cio(), six.text_type)
+    assert isinstance(cio(), str)
     assert cio() == 'ასტრონომიის'
     assert sec['tstunicode'] == 'ასტრონომიის'
 
