@@ -1,21 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-from ..extern import six
 
 import functools
 import types
 import warnings
+import numpy as np
 
 from ..units.core import Unit, UnitsError
 from ..units.quantity import Quantity
 from ..utils import lazyproperty
 from ..utils.exceptions import AstropyUserWarning
+from ..utils.misc import InheritDocstrings
 
 __all__ = ['Constant', 'EMConstant']
 
 
-class ConstantMeta(type):
+class ConstantMeta(InheritDocstrings):
     """Metaclass for the :class:`Constant`. The primary purpose of this is to
     wrap the double-underscore methods of :class:`Quantity` which is the
     superclass of :class:`Constant`.
@@ -36,17 +35,17 @@ class ConstantMeta(type):
             @functools.wraps(meth)
             def wrapper(self, *args, **kwargs):
                 name_lower = self.name.lower()
-                instances = Constant._registry[name_lower]
+                instances = self._registry[name_lower]
                 if not self._checked_units:
-                    for inst in six.itervalues(instances):
+                    for inst in instances.values():
                         try:
                             self.unit.to(inst.unit)
                         except UnitsError:
-                            Constant._has_incompatible_units.add(name_lower)
+                            self._has_incompatible_units.add(name_lower)
                     self._checked_units = True
 
                 if (not self.system and
-                        name_lower in Constant._has_incompatible_units):
+                        name_lower in self._has_incompatible_units):
                     systems = sorted([x for x in instances if x])
                     raise TypeError(
                         'Constant {0!r} does not have physically compatible '
@@ -64,18 +63,17 @@ class ConstantMeta(type):
         exclude = set(['__new__', '__array_finalize__', '__array_wrap__',
                        '__dir__', '__getattr__', '__init__', '__str__',
                        '__repr__', '__hash__', '__iter__', '__getitem__',
-                       '__len__', '__nonzero__'])
-        for attr, value in list(six.iteritems(vars(Quantity))):
+                       '__len__', '__bool__', '__quantity_subclass__'])
+        for attr, value in vars(Quantity).items():
             if (isinstance(value, types.FunctionType) and
                     attr.startswith('__') and attr.endswith('__') and
                     attr not in exclude):
                 d[attr] = wrap(value)
 
-        return super(ConstantMeta, mcls).__new__(mcls, name, bases, d)
+        return super().__new__(mcls, name, bases, d)
 
 
-@six.add_metaclass(ConstantMeta)
-class Constant(Quantity):
+class Constant(Quantity, metaclass=ConstantMeta):
     """A physical or astronomical constant.
 
     These objects are quantities that are meant to represent physical
@@ -84,17 +82,23 @@ class Constant(Quantity):
     _registry = {}
     _has_incompatible_units = set()
 
-    def __new__(cls, abbrev, name, value, unit, uncertainty, reference,
-                system=None):
+    def __new__(cls, abbrev, name, value, unit, uncertainty,
+                reference=None, system=None):
+        if reference is None:
+            reference = getattr(cls, 'default_reference', None)
+            if reference is None:
+                raise TypeError("{} requires a reference.".format(cls))
         name_lower = name.lower()
-        instances = Constant._registry.setdefault(name_lower, {})
+        instances = cls._registry.setdefault(name_lower, {})
+        # By-pass Quantity initialization, since units may not yet be
+        # initialized here, and we store the unit in string form.
+        inst = np.array(value).view(cls)
+
         if system in instances:
-            warnings.warn('Constant {0!r} is already has a definition in the '
-                          '{1!r} system'.format(name, system), AstropyUserWarning)
-
-        inst = super(Constant, cls).__new__(cls, value)
-
-        for c in six.itervalues(instances):
+                warnings.warn('Constant {0!r} already has a definition in the '
+                              '{1!r} system from {2!r} reference'.format(
+                              name, system, reference), AstropyUserWarning)
+        for c in instances.values():
             if system is not None and not hasattr(c.__class__, system):
                 setattr(c, system, inst)
             if c.system is not None and not hasattr(inst.__class__, c.system):
@@ -102,35 +106,44 @@ class Constant(Quantity):
 
         instances[system] = inst
 
+        inst._abbrev = abbrev
+        inst._name = name
+        inst._value = value
+        inst._unit_string = unit
+        inst._uncertainty = uncertainty
+        inst._reference = reference
+        inst._system = system
+
+        inst._checked_units = False
         return inst
 
-    def __init__(self, abbrev, name, value, unit, uncertainty, reference,
-                 system=None):
-        self._abbrev = abbrev
-        self._name = name
-        self._value = value
-        self._unit = unit
-        self._uncertainty = uncertainty
-        self._reference = reference
-        self._system = system
-
-        self._checked_units = False
-
     def __repr__(self):
-        return ('<Constant name={0!r} value={1} error={2} units={3!r} '
-                'reference={4!r}>'.format(self.name, self.value,
+        return ('<{0} name={1!r} value={2} uncertainty={3} unit={4!r} '
+                'reference={5!r}>'.format(self.__class__, self.name, self.value,
                                           self.uncertainty, str(self.unit),
                                           self.reference))
 
     def __str__(self):
         return ('  Name   = {0}\n'
                 '  Value  = {1}\n'
-                '  Error  = {2}\n'
-                '  Units  = {3}\n'
+                '  Uncertainty  = {2}\n'
+                '  Unit  = {3}\n'
                 '  Reference = {4}'.format(self.name, self.value,
                                            self.uncertainty, self.unit,
                                            self.reference))
 
+    def __quantity_subclass__(self, unit):
+        return super().__quantity_subclass__(unit)[0], False
+
+    def copy(self):
+        """
+        Return a copy of this `Constant` instance.  Since they are by
+        definition immutable, this merely returns another reference to
+        ``self``.
+        """
+        return self
+    __deepcopy__ = __copy__ = copy
+
     @property
     def abbrev(self):
         """A typical ASCII text abbreviation of the constant, also generally
@@ -146,10 +159,10 @@ class Constant(Quantity):
         return self._name
 
     @lazyproperty
-    def unit(self):
+    def _unit(self):
         """The unit(s) in which this constant is defined."""
 
-        return Unit(self._unit)
+        return Unit(self._unit_string)
 
     @property
     def uncertainty(self):
@@ -172,14 +185,21 @@ class Constant(Quantity):
 
         return self._system
 
+    def _instance_or_super(self, key):
+        instances = self._registry[self.name.lower()]
+        inst = instances.get(key)
+        if inst is not None:
+            return inst
+        else:
+            return getattr(super(), key)
+
     @property
     def si(self):
         """If the Constant is defined in the SI system return that instance of
         the constant, else convert to a Quantity in the appropriate SI units.
         """
 
-        instances = Constant._registry[self.name.lower()]
-        return instances.get('si') or super(Constant, self).si
+        return self._instance_or_super('si')
 
     @property
     def cgs(self):
@@ -187,8 +207,14 @@ class Constant(Quantity):
         the constant, else convert to a Quantity in the appropriate CGS units.
         """
 
-        instances = Constant._registry[self.name.lower()]
-        return instances.get('cgs') or super(Constant, self).cgs
+        return self._instance_or_super('cgs')
+
+    def __array_finalize__(self, obj):
+        for attr in ('_abbrev', '_name', '_value', '_unit_string',
+                     '_uncertainty', '_reference', '_system'):
+            setattr(self, attr, getattr(obj, attr, None))
+
+        self._checked_units = getattr(obj, '_checked_units', False)
 
 
 class EMConstant(Constant):
@@ -196,8 +222,8 @@ class EMConstant(Constant):
 
     @property
     def cgs(self):
-        """Overridden for EMConstant to raise a `TypeError` emphasizing that
-        there are multiple EM extensions to CGS.
+        """Overridden for EMConstant to raise a `TypeError`
+        emphasizing that there are multiple EM extensions to CGS.
         """
 
         raise TypeError("Cannot convert EM constants to cgs because there "
