@@ -1,7 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import warnings
 
@@ -17,7 +15,6 @@ from ..nddata import support_nddata
 from ..modeling.core import _make_arithmetic_operator, BINARY_OPERATORS
 from ..modeling.core import _CompoundModelMeta
 
-from ..extern.six.moves import range, zip
 
 
 # Disabling all doctests in this module until a better way of handling warnings
@@ -56,9 +53,9 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
         A flag indicating how to handle boundaries:
             * `None`
                 Set the ``result`` values to zero where the kernel
-                extends beyond the edge of the array.
+                extends beyond the edge of the array (default).
             * 'fill'
-                Set values outside the array boundary to ``fill_value`` (default).
+                Set values outside the array boundary to ``fill_value``.
             * 'wrap'
                 Periodic boundary that wrap to the other side of ``array``.
             * 'extend'
@@ -135,7 +132,7 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
     # kernel_internal.  However -- we do want to keep track of what type
     # the input array was so we can cast the result to that at the end
     # if it's a floating point type.  Don't bother with this for lists --
-    # just always push those as np.float.
+    # just always push those as float.
     # It is always necessary to make a copy of kernel (since it is modified),
     # but, if we just so happen to be lucky enough to have the input array
     # have exactly the desired type, we just alias to array_internal
@@ -163,7 +160,7 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
     # Check that the arguments are lists or Numpy arrays
 
     if isinstance(array, list):
-        array_internal = np.array(array, dtype=np.float)
+        array_internal = np.array(array, dtype=float)
         array_dtype = array_internal.dtype
     elif isinstance(array, np.ndarray):
         # Note this won't copy if it doesn't have to -- which is okay
@@ -324,7 +321,7 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
                  fft_pad=None, psf_pad=None, quiet=False,
                  min_wt=0.0, allow_huge=False,
                  fftn=np.fft.fftn, ifftn=np.fft.ifftn,
-                 complex_dtype=np.complex):
+                 complex_dtype=complex):
     """
     Convolve an ndarray with an nd-kernel.  Returns a convolved image with
     ``shape = array.shape``.  Assumes kernel is centered.
@@ -515,8 +512,8 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
 
     # Convert array dtype to complex
     # and ensure that list inputs become arrays
-    array = np.asarray(array, dtype=np.complex)
-    kernel = np.asarray(kernel, dtype=np.complex)
+    array = np.asarray(array, dtype=complex)
+    kernel = np.asarray(kernel, dtype=complex)
 
     # Check that the number of dimensions is compatible
     if array.ndim != kernel.ndim:
@@ -703,6 +700,9 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
         # need to re-zero weights outside of the image (if it is padded, we
         # still don't weight those regions)
         bigimwt[arrayslices] = wtsm.real[arrayslices]
+        # curiously, at the floating-point limit, can get slightly negative numbers
+        # they break the min_wt=0 "flag" and must therefore be removed
+        bigimwt[bigimwt < 0] = 0
     else:
         bigimwt = 1
 
@@ -724,14 +724,11 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
     if interpolate_nan:
         rifft = (ifftn(fftmult)) / bigimwt
         if not np.isscalar(bigimwt):
-            if min_wt > 0.:
-                rifft[bigimwt < min_wt] = np.nan
-            else:
-                # Set anything with no weight to zero (taking into account
-                # slight offsets due to floating-point errors).
-                rifft[bigimwt < 10 * np.finfo(bigimwt.dtype).eps] = 0.0
+            rifft[bigimwt < min_wt] = np.nan
+            if min_wt == 0.0:
+                rifft[bigimwt == 0.0] = 0.0
     else:
-        rifft = ifftn(fftmult)
+        rifft = (ifftn(fftmult))
 
     if preserve_nan:
         rifft[arrayslices][nanmaskarray] = np.nan
