@@ -1,6 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import itertools
 
@@ -14,13 +12,12 @@ from ..kernels import Moffat2DKernel
 
 
 SHAPES_ODD = [[15, 15], [31, 31]]
-SHAPES_EVEN = [[8, 8], [16, 16], [32, 32]]  # FIXME: not used ?!
-NOSHAPE = [[None, None]]
+SHAPES_EVEN = [[8, 8], [16, 16], [32, 32]]
 WIDTHS = [2, 3, 4, 5]
 
 KERNELS = []
 
-for shape in SHAPES_ODD + NOSHAPE:
+for shape in SHAPES_ODD:
     for width in WIDTHS:
 
         KERNELS.append(Gaussian2DKernel(width,
@@ -47,7 +44,7 @@ for shape in SHAPES_ODD + NOSHAPE:
                                       factor=10))
 
 
-class Test2DConvolutions(object):
+class Test2DConvolutions:
 
     @pytest.mark.parametrize('kernel', KERNELS)
     def test_centered_makekernel(self, kernel):
@@ -113,7 +110,7 @@ class Test2DConvolutions(object):
         Compares a small uniform kernel to the Box2DKernel
         """
 
-        kernel1 = np.ones([width, width]) / np.float(width) ** 2
+        kernel1 = np.ones([width, width]) / float(width) ** 2
         kernel2 = Box2DKernel(width, mode='oversample', factor=10)
 
         x = np.zeros(shape)
