@@ -1,13 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import itertools
 
+import pytest
 import numpy as np
 from numpy.testing import assert_almost_equal, assert_allclose
 
-from ...tests.helper import pytest
 from ..convolve import convolve, convolve_fft
 from ..kernels import (
     Gaussian1DKernel, Gaussian2DKernel, Box1DKernel, Box2DKernel,
@@ -17,7 +15,8 @@ from ..kernels import (
 
 from ..utils import KernelSizeError
 from ...modeling.models import Box2D, Gaussian1D, Gaussian2D
-from ...utils.exceptions import AstropyWarning, AstropyUserWarning
+from ...utils.exceptions import AstropyDeprecationWarning
+from ...tests.helper import catch_warnings
 
 try:
     from scipy.ndimage import filters
@@ -34,7 +33,7 @@ KERNEL_TYPES = [Gaussian1DKernel, Gaussian2DKernel,
                 MexicanHat1DKernel, Tophat2DKernel, AiryDisk2DKernel, Ring2DKernel]
 
 
-NUMS = [1, 1., np.float(1.), np.float32(1.), np.float64(1.)]
+NUMS = [1, 1., np.float32(1.), np.float64(1.)]
 
 
 # Test data
@@ -48,7 +47,7 @@ random_data_1D = np.random.rand(61)
 random_data_2D = np.random.rand(61, 61)
 
 
-class TestKernels(object):
+class TestKernels:
     """
     Test class for the built-in convolution kernels.
     """
@@ -82,8 +81,16 @@ class TestKernels(object):
         mexican_kernel_1D = MexicanHat1DKernel(width)
         mexican_kernel_2D = MexicanHat2DKernel(width)
 
-        astropy_1D = convolve(delta_pulse_1D, mexican_kernel_1D, boundary='fill')
-        astropy_2D = convolve(delta_pulse_2D, mexican_kernel_2D, boundary='fill')
+        astropy_1D = convolve(delta_pulse_1D, mexican_kernel_1D, boundary='fill', normalize_kernel=False)
+        astropy_2D = convolve(delta_pulse_2D, mexican_kernel_2D, boundary='fill', normalize_kernel=False)
+
+        with pytest.raises(Exception) as exc:
+            astropy_1D = convolve(delta_pulse_1D, mexican_kernel_1D, boundary='fill', normalize_kernel=True)
+        assert 'sum is close to zero' in exc.value.args[0]
+
+        with pytest.raises(Exception) as exc:
+            astropy_2D = convolve(delta_pulse_2D, mexican_kernel_2D, boundary='fill', normalize_kernel=True)
+        assert 'sum is close to zero' in exc.value.args[0]
 
         # The Laplace of Gaussian filter is an inverted Mexican Hat
         # filter.
@@ -96,7 +103,7 @@ class TestKernels(object):
         assert_almost_equal(astropy_2D, scipy_2D, decimal=5)
 
     @pytest.mark.parametrize(('kernel_type', 'width'), list(itertools.product(KERNEL_TYPES, WIDTHS_ODD)))
-    def test_delta_data(self, kernel_type,  width):
+    def test_delta_data(self, kernel_type, width):
         """
         Test smoothing of an image with a single positive pixel
         """
@@ -108,12 +115,12 @@ class TestKernels(object):
             kernel = kernel_type(width, width * 0.2)
 
         if kernel.dimension == 1:
-            c1 = convolve_fft(delta_pulse_1D, kernel, boundary='fill')
-            c2 = convolve(delta_pulse_1D, kernel, boundary='fill')
+            c1 = convolve_fft(delta_pulse_1D, kernel, boundary='fill', normalize_kernel=False)
+            c2 = convolve(delta_pulse_1D, kernel, boundary='fill', normalize_kernel=False)
             assert_almost_equal(c1, c2, decimal=12)
         else:
-            c1 = convolve_fft(delta_pulse_2D, kernel, boundary='fill')
-            c2 = convolve(delta_pulse_2D, kernel, boundary='fill')
+            c1 = convolve_fft(delta_pulse_2D, kernel, boundary='fill', normalize_kernel=False)
+            c2 = convolve(delta_pulse_2D, kernel, boundary='fill', normalize_kernel=False)
             assert_almost_equal(c1, c2, decimal=12)
 
     @pytest.mark.parametrize(('kernel_type', 'width'), list(itertools.product(KERNEL_TYPES, WIDTHS_ODD)))
@@ -129,12 +136,12 @@ class TestKernels(object):
             kernel = kernel_type(width, width * 0.2)
 
         if kernel.dimension == 1:
-            c1 = convolve_fft(random_data_1D, kernel, boundary='fill')
-            c2 = convolve(random_data_1D, kernel, boundary='fill')
+            c1 = convolve_fft(random_data_1D, kernel, boundary='fill', normalize_kernel=False)
+            c2 = convolve(random_data_1D, kernel, boundary='fill', normalize_kernel=False)
             assert_almost_equal(c1, c2, decimal=12)
         else:
-            c1 = convolve_fft(random_data_2D, kernel, boundary='fill')
-            c2 = convolve(random_data_2D, kernel, boundary='fill')
+            c1 = convolve_fft(random_data_2D, kernel, boundary='fill', normalize_kernel=False)
+            c2 = convolve(random_data_2D, kernel, boundary='fill', normalize_kernel=False)
             assert_almost_equal(c1, c2, decimal=12)
 
     @pytest.mark.parametrize(('width'), WIDTHS_ODD)
@@ -201,7 +208,7 @@ class TestKernels(object):
         """
         gauss = Gaussian1DKernel(3)
         gauss_new = number * gauss
-        assert type(gauss_new) == Gaussian1DKernel
+        assert type(gauss_new) is Gaussian1DKernel
 
     @pytest.mark.parametrize(('number'), NUMS)
     def test_rmultiply_scalar_type(self, number):
@@ -210,7 +217,7 @@ class TestKernels(object):
         """
         gauss = Gaussian1DKernel(3)
         gauss_new = gauss * number
-        assert type(gauss_new) == Gaussian1DKernel
+        assert type(gauss_new) is Gaussian1DKernel
 
     def test_multiply_kernel1d(self):
         """Test that multiplying two 1D kernels raises an exception."""
@@ -263,7 +270,7 @@ class TestKernels(object):
         """
         Check CustomKernel against Box1DKernel.
         """
-        #Define one dimensional array:
+        # Define one dimensional array:
         array = np.ones(5)
         custom = CustomKernel(array)
         custom.normalize()
@@ -277,7 +284,7 @@ class TestKernels(object):
         """
         Check CustomKernel against Box2DKernel.
         """
-        #Define one dimensional array:
+        # Define one dimensional array:
         array = np.ones((5, 5))
         custom = CustomKernel(array)
         custom.normalize()
@@ -292,7 +299,7 @@ class TestKernels(object):
         Check if CustomKernel works with lists.
         """
         custom = CustomKernel([1, 1, 1, 1, 1])
-        assert custom.is_bool == True
+        assert custom.is_bool is True
 
     def test_custom_2D_kernel_list(self):
         """
@@ -301,7 +308,7 @@ class TestKernels(object):
         custom = CustomKernel([[1, 1, 1],
                                [1, 1, 1],
                                [1, 1, 1]])
-        assert custom.is_bool == True
+        assert custom.is_bool is True
 
     def test_custom_1D_kernel_zerosum(self):
         """
@@ -385,6 +392,22 @@ class TestKernels(object):
         gauss = Gaussian2DKernel(3, x_size=10, y_size=10)
         assert gauss.array.shape == (10, 10)
 
+    # https://github.com/astropy/astropy/issues/3605
+    def test_Gaussian2DKernel_rotated(self):
+        with catch_warnings(AstropyDeprecationWarning) as w:
+            Gaussian2DKernel(stddev=10)
+        assert len(w) == 1
+
+        gauss = Gaussian2DKernel(
+            x_stddev=3, y_stddev=1.5, theta=0.7853981633974483,
+            x_size=5, y_size=5)  # rotated 45 deg ccw
+        ans = [[0.02267712, 0.02464785, 0.02029238, 0.01265463, 0.00597762],
+               [0.02464785, 0.03164847, 0.03078144, 0.02267712, 0.01265463],
+               [0.02029238, 0.03078144, 0.03536777, 0.03078144, 0.02029238],
+               [0.01265463, 0.02267712, 0.03078144, 0.03164847, 0.02464785],
+               [0.00597762, 0.01265463, 0.02029238, 0.02464785, 0.02267712]]
+        assert_allclose(gauss, ans, rtol=0.001)  # Rough comparison at 0.1 %
+
     def test_normalize_peak(self):
         """
         Check if normalize works with peak mode.
@@ -416,7 +439,7 @@ class TestKernels(object):
         assert box.separable
 
     @pytest.mark.parametrize(('kernel_type', 'mode'), list(itertools.product(KERNEL_TYPES, MODES)))
-    def test_dicretize_modes(self, kernel_type, mode):
+    def test_discretize_modes(self, kernel_type, mode):
         """
         Check if the different modes result in kernels that work with convolve.
         Use only small kernel width, to make the test pass quickly.
@@ -429,12 +452,12 @@ class TestKernels(object):
             kernel = kernel_type(3, 3 * 0.2)
 
         if kernel.dimension == 1:
-            c1 = convolve_fft(delta_pulse_1D, kernel, boundary='fill')
-            c2 = convolve(delta_pulse_1D, kernel, boundary='fill')
+            c1 = convolve_fft(delta_pulse_1D, kernel, boundary='fill', normalize_kernel=False)
+            c2 = convolve(delta_pulse_1D, kernel, boundary='fill', normalize_kernel=False)
             assert_almost_equal(c1, c2, decimal=12)
         else:
-            c1 = convolve_fft(delta_pulse_2D, kernel, boundary='fill')
-            c2 = convolve(delta_pulse_2D, kernel, boundary='fill')
+            c1 = convolve_fft(delta_pulse_2D, kernel, boundary='fill', normalize_kernel=False)
+            c2 = convolve(delta_pulse_2D, kernel, boundary='fill', normalize_kernel=False)
             assert_almost_equal(c1, c2, decimal=12)
 
     @pytest.mark.parametrize(('width'), WIDTHS_EVEN)
@@ -473,16 +496,6 @@ class TestKernels(object):
             kernel = CustomKernel(np.ones(3))
             kernel.normalize(mode='invalid')
 
-    def test_kernel_normalization_large(self, recwarn):
-        """
-        Test that a warning is issued when the inverse normalization factor
-        is large.
-        """
-        kernel = CustomKernel(np.ones(3) * 1.e-3)
-        kernel.normalize()
-        w = recwarn.pop(AstropyUserWarning)
-        assert issubclass(w.category, AstropyWarning)
-
     def test_kernel1d_int_size(self):
         """
         Test that an error is raised if ``Kernel1D`` ``x_size`` is not
