@@ -1,10 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import division
 
 import numpy as np
 
-from ..modeling.core import Parametric1DModel, Parametric2DModel
-
+from ..modeling.core import FittableModel, custom_model
 
 __all__ = ['discretize_model']
 
@@ -46,7 +44,7 @@ def add_kernel_arrays_1D(array_1, array_2):
 
 def add_kernel_arrays_2D(array_1, array_2):
     """
-    Add two 1D kernel arrays of different size.
+    Add two 2D kernel arrays of different size.
 
     The arrays are added with the centers lying upon each other.
     """
@@ -73,17 +71,24 @@ def add_kernel_arrays_2D(array_1, array_2):
 
 def discretize_model(model, x_range, y_range=None, mode='center', factor=10):
     """
-    Function to evaluate analytical models on a grid.
+    Function to evaluate analytical model functions on a grid.
+
+    So far the function can only deal with pixel coordinates.
 
     Parameters
     ----------
-    model : `~astropy.modeling.ParametricModel`
-        Model to be evaluated.
+    model : `~astropy.modeling.FittableModel` or callable.
+        Analytic model function to be discretized. Callables, which are not an
+        instances of `~astropy.modeling.FittableModel` are passed to
+        `~astropy.modeling.custom_model` and then evaluated.
     x_range : tuple
-        x range in which the model is evaluated.
+        x range in which the model is evaluated. The difference between the
+        upper an lower limit must be a whole number, so that the output array
+        size is well defined.
     y_range : tuple, optional
-        y range in which the model is evaluated.
-        Necessary only for 2D models.
+        y range in which the model is evaluated. The difference between the
+        upper an lower limit must be a whole number, so that the output array
+        size is well defined. Necessary only for 2D models.
     mode : str, optional
         One of the following modes:
             * ``'center'`` (default)
@@ -134,27 +139,46 @@ def discretize_model(model, x_range, y_range=None, mode='center', factor=10):
 
 
     """
-    if isinstance(model, Parametric2DModel) and y_range is None:
-        raise Exception("Please specify y range.")
+    if not callable(model):
+        raise TypeError('Model must be callable.')
+    if not isinstance(model, FittableModel):
+        model = custom_model(model)()
+    ndim = model.n_inputs
+    if ndim > 2:
+        raise ValueError('discretize_model only supports 1-d and 2-d models.')
+
+    if not float(np.diff(x_range)).is_integer():
+        raise ValueError("The difference between the upper an lower limit of"
+                         " 'x_range' must be a whole number.")
+
+    if y_range:
+        if not float(np.diff(y_range)).is_integer():
+            raise ValueError("The difference between the upper an lower limit of"
+                             " 'y_range' must be a whole number.")
+
+    if ndim == 2 and y_range is None:
+        raise ValueError("y range not specified, but model is 2-d")
+    if ndim == 1 and y_range is not None:
+        raise ValueError("y range specified, but model is only 1-d.")
     if mode == "center":
-        if isinstance(model, Parametric1DModel):
+        if ndim == 1:
             return discretize_center_1D(model, x_range)
-        if isinstance(model, Parametric2DModel):
+        elif ndim == 2:
             return discretize_center_2D(model, x_range, y_range)
     elif mode == "linear_interp":
-        if isinstance(model, Parametric1DModel):
+        if ndim == 1:
             return discretize_linear_1D(model, x_range)
-        if isinstance(model, Parametric2DModel):
+        if ndim == 2:
             return discretize_bilinear_2D(model, x_range, y_range)
     elif mode == "oversample":
-        if isinstance(model, Parametric1DModel):
+        if ndim == 1:
             return discretize_oversample_1D(model, x_range, factor)
-        if isinstance(model, Parametric2DModel):
+        if ndim == 2:
             return discretize_oversample_2D(model, x_range, y_range, factor)
     elif mode == "integrate":
-        if isinstance(model, Parametric1DModel):
+        if ndim == 1:
             return discretize_integrate_1D(model, x_range)
-        if isinstance(model, Parametric2DModel):
+        if ndim == 2:
             return discretize_integrate_2D(model, x_range, y_range)
     else:
         raise DiscretizationError('Invalid mode.')
@@ -245,11 +269,7 @@ def discretize_integrate_1D(model, x_range):
     """
     Discretize model by integrating numerically the model over the bin.
     """
-    try:
-        from scipy.integrate import quad
-    except ImportError:
-        raise Exception("Mode 'integrate' requires scipy.")
-
+    from scipy.integrate import quad
     # Set up grid
     x = np.arange(x_range[0] - 0.5, x_range[1] + 0.5)
     values = np.array([])
@@ -264,11 +284,7 @@ def discretize_integrate_2D(model, x_range, y_range):
     """
     Discretize model by integrating the model over the pixel.
     """
-    try:
-        from scipy.integrate import dblquad
-    except ImportError:
-        raise Exception("Mode 'integrate' requires scipy.")
-
+    from scipy.integrate import dblquad
     # Set up grid
     x = np.arange(x_range[0] - 0.5, x_range[1] + 0.5)
     y = np.arange(y_range[0] - 0.5, y_range[1] + 0.5)
@@ -277,6 +293,6 @@ def discretize_integrate_2D(model, x_range, y_range):
     # Integrate over all pixels
     for i in range(x.size - 1):
         for j in range(y.size - 1):
-            values[j, i] = dblquad(model, x[i], x[i + 1],
+            values[j, i] = dblquad(lambda y, x: model(x, y), x[i], x[i + 1],
                                    lambda x: y[j], lambda x: y[j + 1])[0]
     return values
