@@ -5,19 +5,15 @@
 This module contains the fundamental classes used for representing
 coordinates in astropy.
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import math
 from collections import namedtuple
 
 import numpy as np
 
-from ..extern import six
 from . import angle_utilities as util
 from .. import units as u
 from ..utils import isiterable
-from ..utils.compat import NUMPY_LT_1_14_1
 
 __all__ = ['Angle', 'Latitude', 'Longitude']
 
@@ -93,7 +89,7 @@ class Angle(u.SpecificTypeQuantity):
             if isinstance(angle, tuple):
                 angle = cls._tuple_to_float(angle, unit)
 
-            elif isinstance(angle, six.string_types):
+            elif isinstance(angle, str):
                 angle, angle_unit = util.parse_angle(angle, unit)
                 if angle_unit is None:
                     angle_unit = unit
@@ -110,8 +106,7 @@ class Angle(u.SpecificTypeQuantity):
                        angle.dtype.kind not in 'SUVO')):
                 angle = [Angle(x, unit, copy=False) for x in angle]
 
-        return super(Angle, cls).__new__(cls, angle, unit, dtype=dtype,
-                                         copy=copy)
+        return super().__new__(cls, angle, unit, dtype=dtype, copy=copy)
 
     @staticmethod
     def _tuple_to_float(angle, unit):
@@ -133,7 +128,7 @@ class Angle(u.SpecificTypeQuantity):
         return u.hourangle if unit is u.hour else unit
 
     def _set_unit(self, unit):
-        super(Angle, self)._set_unit(self._convert_unit_to_angle_unit(unit))
+        super()._set_unit(self._convert_unit_to_angle_unit(unit))
 
     @property
     def hour(self):
@@ -431,26 +426,19 @@ class Angle(u.SpecificTypeQuantity):
             ok &= np.all(self < Angle(upper))
         return bool(ok)
 
-    def _str_helper(self, format=None):
-        if self.isscalar:
-            return self.to_string(format=format)
-
-        if NUMPY_LT_1_14_1:
-            def formatter(x):
-                return x.to_string(format=format)
-        else:
-            # In numpy 1.14.1, array2print formatters get passed plain numpy scalars instead
-            # of subclass array scalars, so we need to recreate an array scalar.
-            def formatter(x):
-                return self._new_view(x).to_string(format=format)
-
-        return np.array2string(self, formatter={'all': formatter})
-
     def __str__(self):
-        return self._str_helper()
+        if self.isscalar:
+            return str(self.to_string())
+        else:
+            return np.array2string(self, formatter={'all': lambda x: x.to_string()})
 
     def _repr_latex_(self):
-        return self._str_helper(format='latex')
+        if self.isscalar:
+            return self.to_string(format='latex')
+        else:
+            # Need to do a magic incantation to convert to str.  Regular str
+            # or array2string causes all backslashes to get doubled.
+            return np.array2string(self, formatter={'all': lambda x: x.to_string(format='latex')})
 
 
 def _no_angle_subclass(obj):
@@ -515,7 +503,7 @@ class Latitude(Angle):
         # Forbid creating a Lat from a Long.
         if isinstance(angle, Longitude):
             raise TypeError("A Latitude angle cannot be created from a Longitude angle")
-        self = super(Latitude, cls).__new__(cls, angle, unit=unit, **kwargs)
+        self = super().__new__(cls, angle, unit=unit, **kwargs)
         self._validate_angles()
         return self
 
@@ -541,15 +529,15 @@ class Latitude(Angle):
             raise TypeError("A Longitude angle cannot be assigned to a Latitude angle")
         # first check bounds
         self._validate_angles(value)
-        super(Latitude, self).__setitem__(item, value)
+        super().__setitem__(item, value)
 
     # Any calculation should drop to Angle
     def __array_wrap__(self, obj, context=None):
-        obj = super(Angle, self).__array_wrap__(obj, context=context)
+        obj = super().__array_wrap__(obj, context=context)
         return _no_angle_subclass(obj)
 
     def __array_ufunc__(self, *args, **kwargs):
-        results = super(Latitude, self).__array_ufunc__(*args, **kwargs)
+        results = super().__array_ufunc__(*args, **kwargs)
         return _no_angle_subclass(results)
 
 
@@ -621,7 +609,7 @@ class Longitude(Angle):
         if isinstance(angle, Latitude):
             raise TypeError("A Longitude angle cannot be created from "
                             "a Latitude angle.")
-        self = super(Longitude, cls).__new__(cls, angle, unit=unit, **kwargs)
+        self = super().__new__(cls, angle, unit=unit, **kwargs)
         if wrap_angle is None:
             wrap_angle = getattr(angle, 'wrap_angle', self._default_wrap_angle)
         self.wrap_angle = wrap_angle
@@ -631,7 +619,7 @@ class Longitude(Angle):
         # Forbid assigning a Lat to a Long.
         if isinstance(value, Latitude):
             raise TypeError("A Latitude angle cannot be assigned to a Longitude angle")
-        super(Longitude, self).__setitem__(item, value)
+        super().__setitem__(item, value)
         self._wrap_internal()
 
     def _wrap_internal(self):
@@ -651,7 +639,7 @@ class Longitude(Angle):
         if np.any(self_angle < wrap_angle_floor) or np.any(self_angle >= wrap_angle):
             wrapped = np.mod(self_angle - wrap_angle, a360) + wrap_angle_floor
             value = u.Quantity(wrapped, self.unit)
-            super(Longitude, self).__setitem__((), value)
+            super().__setitem__((), value)
 
     @property
     def wrap_angle(self):
@@ -663,15 +651,15 @@ class Longitude(Angle):
         self._wrap_internal()
 
     def __array_finalize__(self, obj):
-        super(Longitude, self).__array_finalize__(obj)
+        super().__array_finalize__(obj)
         self._wrap_angle = getattr(obj, '_wrap_angle',
                                    self._default_wrap_angle)
 
     # Any calculation should drop to Angle
     def __array_wrap__(self, obj, context=None):
-        obj = super(Angle, self).__array_wrap__(obj, context=context)
+        obj = super().__array_wrap__(obj, context=context)
         return _no_angle_subclass(obj)
 
     def __array_ufunc__(self, *args, **kwargs):
-        results = super(Longitude, self).__array_ufunc__(*args, **kwargs)
+        results = super().__array_ufunc__(*args, **kwargs)
         return _no_angle_subclass(results)
