@@ -1,36 +1,21 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, unicode_literals, division,
-                        print_function)
 
 import numpy as np
 
 from ... import units as u
+from ...utils.decorators import format_doc
 from .. import representation as r
-from ..baseframe import BaseCoordinateFrame, RepresentationMapping
+from ..baseframe import BaseCoordinateFrame, RepresentationMapping, base_doc
 from ..attributes import (Attribute, TimeAttribute,
                           QuantityAttribute, EarthLocationAttribute)
 
+__all__ = ['AltAz']
+
+
 _90DEG = 90*u.deg
 
-
-class AltAz(BaseCoordinateFrame):
-    """
-    A coordinate or frame in the Altitude-Azimuth system (Horizontal
-    coordinates).  Azimuth is oriented East of North (i.e., N=0, E=90 degrees).
-
-    This frame is assumed to *include* refraction effects if the ``pressure``
-    frame attribute is non-zero.
-
-    The frame attributes are listed under **Other Parameters**, which are
-    necessary for transforming from AltAz to some other system.
-
-    Parameters
-    ----------
-    representation : `BaseRepresentation` or None
-        A representation object or None to have no data (or use the other
-        keywords)
-
+doc_components = """
     az : `Angle`, optional, must be keyword
         The Azimuth for this object (``alt`` must also be given and
         ``representation`` must be None).
@@ -47,20 +32,9 @@ class AltAz(BaseCoordinateFrame):
         The proper motion in altitude for this object (``pm_az_cosalt`` must
         also be given).
     radial_velocity : :class:`~astropy.units.Quantity`, optional, must be keyword
-        The radial velocity of this object.
-
-    copy : bool, optional
-        If `True` (default), make copies of the input coordinate arrays.
-        Can only be passed in as a keyword argument.
-
-    differential_cls : `BaseDifferential`, dict, optional
-        A differential class or dictionary of differential classes (currently
-        only a velocity differential with key 's' is supported). This sets
-        the expected input differential class, thereby changing the expected
-        keyword arguments of the data passed in. For example, passing
-        ``differential_cls=CartesianDifferential`` will make the classes
-        expect velocity data with the argument names ``v_x, v_y, v_z``.
+        The radial velocity of this object."""
 
+doc_footer = """
     Other parameters
     ----------------
     obstime : `~astropy.time.Time`
@@ -78,9 +52,9 @@ class AltAz(BaseCoordinateFrame):
     temperature : `~astropy.units.Quantity`
         The ground-level temperature as an `~astropy.units.Quantity` in
         deg C.  This is necessary for performing refraction corrections.
-    relative_humidity`` : `~astropy.units.Quantity` or number.
-        The relative humidity as a dimensionless quantity between 0 to 1.
-        This is necessary for performing refraction corrections.
+    relative_humidity`` : numeric
+        The relative humidity as a number from 0 to 1.  This is necessary for
+        performing refraction corrections.
     obswl : `~astropy.units.Quantity`
         The average wavelength of observations as an `~astropy.units.Quantity`
          with length units.  This is necessary for performing refraction
@@ -91,40 +65,31 @@ class AltAz(BaseCoordinateFrame):
     The refraction model is based on that implemented in ERFA, which is fast
     but becomes inaccurate for altitudes below about 5 degrees.  Near and below
     altitudes of 0, it can even give meaningless answers, and in this case
-    transforming to AltAz and back to another frame can give highly discrepant
+    transforming to AltAz and back to another frame can give highly discrepent
     results.  For much better numerical stability, leaving the ``pressure`` at
     ``0`` (the default), disabling the refraction correction (yielding
     "topocentric" horizontal coordinates).
+    """
 
+@format_doc(base_doc, components=doc_components, footer=doc_footer)
+class AltAz(BaseCoordinateFrame):
+    """
+    A coordinate or frame in the Altitude-Azimuth system (Horizontal
+    coordinates).  Azimuth is oriented East of North (i.e., N=0, E=90 degrees).
+
+    This frame is assumed to *include* refraction effects if the ``pressure``
+    frame attribute is non-zero.
+
+    The frame attributes are listed under **Other Parameters**, which are
+    necessary for transforming from AltAz to some other system.
     """
 
     frame_specific_representation_info = {
         r.SphericalRepresentation: [
             RepresentationMapping('lon', 'az'),
             RepresentationMapping('lat', 'alt')
-        ],
-        r.SphericalCosLatDifferential: [
-            RepresentationMapping('d_lon_coslat', 'pm_az_cosalt', u.mas/u.yr),
-            RepresentationMapping('d_lat', 'pm_alt', u.mas/u.yr),
-            RepresentationMapping('d_distance', 'radial_velocity', u.km/u.s),
-        ],
-        r.SphericalDifferential: [
-            RepresentationMapping('d_lon', 'pm_az', u.mas/u.yr),
-            RepresentationMapping('d_lat', 'pm_alt', u.mas/u.yr),
-            RepresentationMapping('d_distance', 'radial_velocity', u.km/u.s)
-        ],
-        r.CartesianDifferential: [
-            RepresentationMapping('d_x', 'v_x', u.km/u.s),
-            RepresentationMapping('d_y', 'v_y', u.km/u.s),
-            RepresentationMapping('d_z', 'v_z', u.km/u.s),
-        ],
+        ]
     }
-    frame_specific_representation_info[r.UnitSphericalRepresentation] = \
-        frame_specific_representation_info[r.SphericalRepresentation]
-    frame_specific_representation_info[r.UnitSphericalCosLatDifferential] = \
-        frame_specific_representation_info[r.SphericalCosLatDifferential]
-    frame_specific_representation_info[r.UnitSphericalDifferential] = \
-        frame_specific_representation_info[r.SphericalDifferential]
 
     default_representation = r.SphericalRepresentation
     default_differential = r.SphericalCosLatDifferential
@@ -133,11 +98,11 @@ class AltAz(BaseCoordinateFrame):
     location = EarthLocationAttribute(default=None)
     pressure = QuantityAttribute(default=0, unit=u.hPa)
     temperature = QuantityAttribute(default=0, unit=u.deg_C)
-    relative_humidity = QuantityAttribute(default=0, unit=u.dimensionless_unscaled)
+    relative_humidity = Attribute(default=0)
     obswl = QuantityAttribute(default=1*u.micron, unit=u.micron)
 
     def __init__(self, *args, **kwargs):
-        super(AltAz, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @property
     def secz(self):
