@@ -4,8 +4,6 @@
 This module contains functions/values used repeatedly in different modules of
 the ``builtin_frames`` package.
 """
-from __future__ import (absolute_import, unicode_literals, division,
-                        print_function)
 
 import warnings
 
@@ -17,7 +15,6 @@ from ...time import Time
 from ...utils import iers
 from ...utils.exceptions import AstropyWarning
 
-from ...extern.six.moves import range
 
 # The UTC time scale is not properly defined prior to 1960, so Time('B1950',
 # scale='utc') will emit a warning. Instead, we use Time('B1950', scale='tai')
@@ -122,13 +119,7 @@ def norm(p):
     """
     Normalise a p-vector.
     """
-    if np.__version__ == '1.14.0':
-        # there is a bug in numpy v1.14.0 (fixed in 1.14.1) that causes
-        # this einsum call to break with the default of optimize=True
-        # see https://github.com/astropy/astropy/issues/7051
-        return p / np.sqrt(np.einsum('...i,...i', p, p, optimize=False))[..., np.newaxis]
-    else:
-        return p / np.sqrt(np.einsum('...i,...i', p, p))[..., np.newaxis]
+    return p/np.sqrt(np.einsum('...i,...i', p, p))[..., np.newaxis]
 
 
 def get_cip(jd1, jd2):
