@@ -4,9 +4,7 @@ This module contains convenience functions for retrieving solar system
 ephemerides from jplephem.
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
+from urllib.parse import urlparse
 from collections import OrderedDict
 
 import numpy as np
@@ -22,12 +20,10 @@ from ..constants import c as speed_of_light
 from .representation import CartesianRepresentation
 from .orbital_elements import calc_moon
 from .builtin_frames import GCRS, ICRS
-from .builtin_frames.utils import get_jd12, cartrepr_from_matmul
-from .. import _erfa
-from ..extern import six
+from .builtin_frames.utils import get_jd12
 
 __all__ = ["get_body", "get_moon", "get_body_barycentric",
-           "solar_system_ephemeris"]
+           "get_body_barycentric_posvel", "solar_system_ephemeris"]
 
 
 DEFAULT_JPL_EPHEMERIS = 'de430'
@@ -38,7 +34,7 @@ BODY_NAME_TO_KERNEL_SPEC = OrderedDict(
                                        ('mercury', [(0, 1), (1, 199)]),
                                        ('venus', [(0, 2), (2, 299)]),
                                        ('earth-moon-barycenter', [(0, 3)]),
-                                       ('earth',  [(0, 3), (3, 399)]),
+                                       ('earth', [(0, 3), (3, 399)]),
                                        ('moon', [(0, 3), (3, 301)]),
                                        ('mars', [(0, 4)]),
                                        ('jupiter', [(0, 5)]),
@@ -53,7 +49,6 @@ PLAN94_BODY_NAME_TO_PLANET_INDEX = OrderedDict(
     (('mercury', 1),
      ('venus', 2),
      ('earth-moon-barycenter', 3),
-     ('moon', 301),
      ('mars', 4),
      ('jupiter', 5),
      ('saturn', 6),
@@ -74,7 +69,7 @@ If needed, the ephemeris file will be downloaded (and cached).
 
 One can check which bodies are covered by a given ephemeris using::
     >>> solar_system_ephemeris.bodies
-    ('earth', 'sun', 'mercury', 'venus', 'earth-moon-barycenter', 'moon', 'mars', 'jupiter', 'saturn', 'uranus', 'neptune')
+    ('earth', 'sun', 'moon', 'mercury', 'venus', 'earth-moon-barycenter', 'mars', 'jupiter', 'saturn', 'uranus', 'neptune')
 """[1:-1]
 
 
@@ -86,7 +81,7 @@ class solar_system_ephemeris(ScienceState):
     - 'builtin': polynomial approximations to the orbital elements.
     - 'de430' or 'de432s': short-cuts for recent JPL dynamical models.
     - 'jpl': Alias for the default JPL ephemeris (currently, 'de430').
-    - URL: (str) The url to a SPK ephemeris in SPICE binary (.bst) format.
+    - URL: (str) The url to a SPK ephemeris in SPICE binary (.bsp) format.
     - `None`: Ensure an Exception is raised without an explicit ephemeris.
 
     The default is 'builtin', which uses the ``epv00`` and ``plan94``
@@ -110,6 +105,9 @@ class solar_system_ephemeris(ScienceState):
 
     @classmethod
     def validate(cls, value):
+        # make no changes if value is None
+        if value is None:
+            return cls._value
         # Set up Kernel; if the file is not in cache, this will download it.
         cls.get_kernel(value)
         return value
@@ -137,7 +135,7 @@ class solar_system_ephemeris(ScienceState):
         if cls._value is None:
             return None
         if cls._value.lower() == 'builtin':
-            return (('earth', 'sun') +
+            return (('earth', 'sun', 'moon') +
                     tuple(PLAN94_BODY_NAME_TO_PLANET_INDEX.keys()))
         else:
             return tuple(BODY_NAME_TO_KERNEL_SPEC.keys())
@@ -159,8 +157,8 @@ def _get_kernel(value):
                  '/spk/planets/{:s}.bsp'.format(value.lower()))
     else:
         try:
-            six.moves.urllib.parse.urlparse(value)
-        except:
+            urlparse(value)
+        except Exception:
             raise ValueError('{} was not one of the standard strings and '
                              'could not be parsed as a URL'.format(value))
 
@@ -174,6 +172,167 @@ def _get_kernel(value):
     return SPK.open(download_file(value, cache=True))
 
 
+def _get_body_barycentric_posvel(body, time, ephemeris=None,
+                                 get_velocity=True):
+    """Calculate the barycentric position (and velocity) of a solar system body.
+
+    Parameters
+    ----------
+    body : str or other
+        The solar system body for which to calculate positions.  Can also be a
+        kernel specifier (list of 2-tuples) if the ``ephemeris`` is a JPL
+        kernel.
+    time : `~astropy.time.Time`
+        Time of observation.
+    ephemeris : str, optional
+        Ephemeris to use.  By default, use the one set with
+        ``astropy.coordinates.solar_system_ephemeris.set``
+    get_velocity : bool, optional
+        Whether or not to calculate the velocity as well as the position.
+
+    Returns
+    -------
+    position : `~astropy.coordinates.CartesianRepresentation` or tuple
+        Barycentric (ICRS) position or tuple of position and velocity.
+
+    Notes
+    -----
+    No velocity can be calculated with the built-in ephemeris for the Moon.
+
+    Whether or not velocities are calculated makes little difference for the
+    built-in ephemerides, but for most JPL ephemeris files, the execution time
+    roughly doubles.
+    """
+
+    if ephemeris is None:
+        ephemeris = solar_system_ephemeris.get()
+        if ephemeris is None:
+            raise ValueError(_EPHEMERIS_NOTE)
+        kernel = solar_system_ephemeris.kernel
+    else:
+        kernel = _get_kernel(ephemeris)
+
+    jd1, jd2 = get_jd12(time, 'tdb')
+    if kernel is None:
+        body = body.lower()
+        earth_pv_helio, earth_pv_bary = erfa.epv00(jd1, jd2)
+        if body == 'earth':
+            body_pv_bary = earth_pv_bary
+
+        elif body == 'moon':
+            if get_velocity:
+                raise KeyError("the Moon's velocity cannot be calculated with "
+                               "the '{0}' ephemeris.".format(ephemeris))
+            return calc_moon(time).cartesian
+
+        else:
+            sun_pv_bary = earth_pv_bary - earth_pv_helio
+            if body == 'sun':
+                body_pv_bary = sun_pv_bary
+            else:
+                try:
+                    body_index = PLAN94_BODY_NAME_TO_PLANET_INDEX[body]
+                except KeyError:
+                    raise KeyError("{0}'s position and velocity cannot be "
+                                   "calculated with the '{1}' ephemeris."
+                                   .format(body, ephemeris))
+                body_pv_helio = erfa.plan94(jd1, jd2, body_index)
+                body_pv_bary = body_pv_helio + sun_pv_bary
+
+        body_pos_bary = CartesianRepresentation(
+            body_pv_bary[..., 0, :], unit=u.au, xyz_axis=-1, copy=False)
+        if get_velocity:
+            body_vel_bary = CartesianRepresentation(
+                body_pv_bary[..., 1, :], unit=u.au/u.day, xyz_axis=-1,
+                copy=False)
+
+    else:
+        if isinstance(body, str):
+            # Look up kernel chain for JPL ephemeris, based on name
+            try:
+                kernel_spec = BODY_NAME_TO_KERNEL_SPEC[body.lower()]
+            except KeyError:
+                raise KeyError("{0}'s position cannot be calculated with "
+                               "the {1} ephemeris.".format(body, ephemeris))
+        else:
+            # otherwise, assume the user knows what their doing and intentionally
+            # passed in a kernel chain
+            kernel_spec = body
+
+        # jplephem cannot handle multi-D arrays, so convert to 1D here.
+        jd1_shape = getattr(jd1, 'shape', ())
+        if len(jd1_shape) > 1:
+            jd1, jd2 = jd1.ravel(), jd2.ravel()
+        # Note that we use the new jd1.shape here to create a 1D result array.
+        # It is reshaped below.
+        body_posvel_bary = np.zeros((2 if get_velocity else 1, 3) +
+                                     getattr(jd1, 'shape', ()))
+        for pair in kernel_spec:
+            spk = kernel[pair]
+            if spk.data_type == 3:
+                # Type 3 kernels contain both position and velocity.
+                posvel = spk.compute(jd1, jd2)
+                if get_velocity:
+                    body_posvel_bary += posvel.reshape(body_posvel_bary.shape)
+                else:
+                    body_posvel_bary[0] += posvel[:4]
+            else:
+                # spk.generate first yields the position and then the
+                # derivative. If no velocities are desired, body_posvel_bary
+                # has only one element and thus the loop ends after a single
+                # iteration, avoiding the velocity calculation.
+                for body_p_or_v, p_or_v in zip(body_posvel_bary,
+                                               spk.generate(jd1, jd2)):
+                    body_p_or_v += p_or_v
+
+        body_posvel_bary.shape = body_posvel_bary.shape[:2] + jd1_shape
+        body_pos_bary = CartesianRepresentation(body_posvel_bary[0],
+                                                unit=u.km, copy=False)
+        if get_velocity:
+            body_vel_bary = CartesianRepresentation(body_posvel_bary[1],
+                                                    unit=u.km/u.day, copy=False)
+
+    return (body_pos_bary, body_vel_bary) if get_velocity else body_pos_bary
+
+
+def get_body_barycentric_posvel(body, time, ephemeris=None):
+    """Calculate the barycentric position and velocity of a solar system body.
+
+    Parameters
+    ----------
+    body : str or other
+        The solar system body for which to calculate positions.  Can also be a
+        kernel specifier (list of 2-tuples) if the ``ephemeris`` is a JPL
+        kernel.
+    time : `~astropy.time.Time`
+        Time of observation.
+    ephemeris : str, optional
+        Ephemeris to use.  By default, use the one set with
+        ``astropy.coordinates.solar_system_ephemeris.set``
+
+    Returns
+    -------
+    position, velocity : tuple of `~astropy.coordinates.CartesianRepresentation`
+        Tuple of barycentric (ICRS) position and velocity.
+
+    See also
+    --------
+    get_body_barycentric : to calculate position only.
+        This is faster by about a factor two for JPL kernels, but has no
+        speed advantage for the built-in ephemeris.
+
+    Notes
+    -----
+    The velocity cannot be calculated for the Moon.  To just get the position,
+    use :func:`~astropy.coordinates.get_body_barycentric`.
+
+    """
+    return _get_body_barycentric_posvel(body, time, ephemeris)
+
+
+get_body_barycentric_posvel.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
+
+
 def get_body_barycentric(body, time, ephemeris=None):
     """Calculate the barycentric position of a solar system body.
 
@@ -191,67 +350,19 @@ def get_body_barycentric(body, time, ephemeris=None):
 
     Returns
     -------
-    cartesian_position : `~astropy.coordinates.CartesianRepresentation`
+    position : `~astropy.coordinates.CartesianRepresentation`
         Barycentric (ICRS) position of the body in cartesian coordinates
 
+    See also
+    --------
+    get_body_barycentric_posvel : to calculate both position and velocity.
+
     Notes
     -----
     """
+    return _get_body_barycentric_posvel(body, time, ephemeris,
+                                        get_velocity=False)
 
-    if ephemeris is None:
-        ephemeris = solar_system_ephemeris.get()
-        if ephemeris is None:
-            raise ValueError(_EPHEMERIS_NOTE)
-        kernel = solar_system_ephemeris.kernel
-    else:
-        kernel = _get_kernel(ephemeris)
-
-    jd1, jd2 = get_jd12(time, 'tdb')
-    if kernel is None:
-        body = body.lower()
-        earth_pv_helio, earth_pv_bary = erfa.epv00(jd1, jd2)
-        if body == 'earth':
-            cartesian_position_body = earth_pv_bary[..., 0, :]
-        elif body == 'moon':
-            cartesian_position_body = calc_moon(time).cartesian.xyz.to(u.au).value
-            cartesian_position_body = np.rollaxis(cartesian_position_body, 0,
-                                                  cartesian_position_body.ndim)
-        else:
-            sun_bary = earth_pv_bary[..., 0, :] - earth_pv_helio[..., 0, :]
-            if body == 'sun':
-                cartesian_position_body = sun_bary
-            else:
-                try:
-                    body_index = PLAN94_BODY_NAME_TO_PLANET_INDEX[body]
-                except KeyError:
-                    raise KeyError("{0}'s position cannot be calculated with "
-                                   "the '{1}' ephemeris."
-                                   .format(body, ephemeris))
-                body_pv_helio = erfa.plan94(jd1, jd2, body_index)
-                cartesian_position_body = body_pv_helio[..., 0, :] + sun_bary
-
-        barycen_to_body_vector = u.Quantity(
-            np.rollaxis(cartesian_position_body, -1, 0), u.au)
-
-    else:
-        if isinstance(body, six.string_types):
-            # Look up kernel chain for JPL ephemeris, based on name
-            try:
-                kernel_spec = BODY_NAME_TO_KERNEL_SPEC[body.lower()]
-            except KeyError:
-                raise KeyError("{0}'s position cannot be calculated with "
-                               "the {1} ephemeris.".format(body, ephemeris))
-        else:
-            # otherwise, assume the user knows what their doing and intentionally
-            # passed in a kernel chain
-            kernel_spec = body
-
-        cartesian_position_body = sum([kernel[pair].compute(jd1, jd2)
-                                       for pair in kernel_spec])
-
-        barycen_to_body_vector = u.Quantity(cartesian_position_body, unit=u.km)
-
-    return CartesianRepresentation(barycen_to_body_vector)
 
 get_body_barycentric.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
 
@@ -293,8 +404,7 @@ def _get_apparent_body_position(body, time, ephemeris):
     earth_loc = get_body_barycentric('earth', time, ephemeris)
     while np.any(np.fabs(delta_light_travel_time) > 1.0e-8*u.s):
         body_loc = get_body_barycentric(body, emitted_time, ephemeris)
-        earth_body_vector = body_loc.xyz - earth_loc.xyz
-        earth_distance = np.sqrt(np.sum(earth_body_vector**2, axis=0))
+        earth_distance = (body_loc - earth_loc).norm()
         delta_light_travel_time = (light_travel_time -
                                    earth_distance/speed_of_light)
         light_travel_time = earth_distance/speed_of_light
@@ -302,13 +412,15 @@ def _get_apparent_body_position(body, time, ephemeris):
 
     return get_body_barycentric(body, emitted_time, ephemeris)
 
+
 _get_apparent_body_position.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
 
 
 def get_body(body, time, location=None, ephemeris=None):
     """
     Get a `~astropy.coordinates.SkyCoord` for a solar system body as observed
-    from a location on Earth.
+    from a location on Earth in the `~astropy.coordinates.GCRS` reference
+    system.
 
     Parameters
     ----------
@@ -348,13 +460,15 @@ def get_body(body, time, location=None, ephemeris=None):
         gcrs = icrs.transform_to(GCRS(obstime=time))
     return SkyCoord(gcrs)
 
+
 get_body.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
 
 
 def get_moon(time, location=None, ephemeris=None):
     """
     Get a `~astropy.coordinates.SkyCoord` for the Earth's Moon as observed
-    from a location on Earth.
+    from a location on Earth in the `~astropy.coordinates.GCRS` reference
+    system.
 
     Parameters
     ----------
@@ -371,7 +485,7 @@ def get_moon(time, location=None, ephemeris=None):
     Returns
     -------
     skycoord : `~astropy.coordinates.SkyCoord`
-        Coordinate for the Moon
+        GCRS Coordinate for the Moon
 
     Notes
     -----
@@ -379,6 +493,7 @@ def get_moon(time, location=None, ephemeris=None):
 
     return get_body('moon', time, location=location, ephemeris=ephemeris)
 
+
 get_moon.__doc__ += indent(_EPHEMERIS_NOTE)[4:]
 
 
@@ -388,5 +503,6 @@ def _apparent_position_in_true_coordinates(skycoord):
     are defined w.r.t to the true equinox and poles of the Earth
     """
     jd1, jd2 = get_jd12(skycoord.obstime, 'tt')
-    _, _, _, _, _, _, _, rbpn = _erfa.pn00a(jd1, jd2)
-    return SkyCoord(skycoord.frame.realize_frame(cartrepr_from_matmul(rbpn, skycoord)))
+    _, _, _, _, _, _, _, rbpn = erfa.pn00a(jd1, jd2)
+    return SkyCoord(skycoord.frame.realize_frame(
+        skycoord.cartesian.transform(rbpn)))
