@@ -1,21 +1,17 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-# TEST_UNICODE_LITERALS
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 """Test initalization and other aspects of Angle and subclasses"""
 
 import pytest
 import numpy as np
-from numpy.testing import assert_allclose, assert_array_equal
+from numpy.testing.utils import assert_allclose, assert_array_equal
 
 from ..angles import Longitude, Latitude, Angle
 from ... import units as u
-from ..errors import (IllegalSecondError, IllegalMinuteError, IllegalHourError,
-                      IllegalSecondWarning, IllegalMinuteWarning)
+from ..errors import IllegalSecondError, IllegalMinuteError, IllegalHourError
 
 
 def test_create_angles():
@@ -465,56 +461,48 @@ def test_negative_zero_hm():
 
 def test_negative_sixty_hm():
     # Test for HM parser
-    with pytest.warns(IllegalMinuteWarning):
-        a = Angle('-00:60', u.hour)
+    a = Angle('-00:60', u.hour)
     assert_allclose(a.hour, -1.)
 
 
 def test_plus_sixty_hm():
     # Test for HM parser
-    with pytest.warns(IllegalMinuteWarning):
-        a = Angle('00:60', u.hour)
+    a = Angle('00:60', u.hour)
     assert_allclose(a.hour, 1.)
 
 
 def test_negative_fifty_nine_sixty_dms():
     # Test for DMS parser
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('-00:59:60', u.deg)
+    a = Angle('-00:59:60', u.deg)
     assert_allclose(a.degree, -1.)
 
 
 def test_plus_fifty_nine_sixty_dms():
     # Test for DMS parser
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('+00:59:60', u.deg)
+    a = Angle('+00:59:60', u.deg)
     assert_allclose(a.degree, 1.)
 
 
 def test_negative_sixty_dms():
     # Test for DMS parser
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('-00:00:60', u.deg)
+    a = Angle('-00:00:60', u.deg)
     assert_allclose(a.degree, -1. / 60.)
 
 
 def test_plus_sixty_dms():
     # Test for DMS parser
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('+00:00:60', u.deg)
+    a = Angle('+00:00:60', u.deg)
     assert_allclose(a.degree, 1. / 60.)
 
 
 def test_angle_to_is_angle():
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('00:00:60', u.deg)
+    a = Angle('00:00:60', u.deg)
     assert isinstance(a, Angle)
     assert isinstance(a.to(u.rad), Angle)
 
 
 def test_angle_to_quantity():
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('00:00:60', u.deg)
+    a = Angle('00:00:60', u.deg)
     q = u.Quantity(a)
     assert isinstance(q, u.Quantity)
     assert q.unit is u.deg
@@ -533,8 +521,7 @@ def test_quantity_to_angle():
 
 
 def test_angle_string():
-    with pytest.warns(IllegalSecondWarning):
-        a = Angle('00:00:60', u.deg)
+    a = Angle('00:00:60', u.deg)
     assert str(a) == '0d01m00s'
     a = Angle('-00:00:10', u.hour)
     assert str(a) == '-0h00m10s'
@@ -859,7 +846,6 @@ def test_wrap_at_without_new():
     l = np.concatenate([l1, l2])
     assert l._wrap_angle is not None
 
-
 def test__str__():
     """
     Check the __str__ method used in printing the Angle
@@ -880,7 +866,6 @@ def test__str__():
     bigarrangle = Angle(np.ones(10000), u.deg)
     assert '...' in bigarrangle.__str__()
 
-
 def test_repr_latex():
     """
     Check the _repr_latex_ method, used primarily by IPython notebooks
