@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 """
 This is the APE5 coordinates API document re-written to work as a series of test
@@ -16,6 +14,7 @@ deviations from the original APE5 plan.
 
 import pytest
 import numpy as np
+from numpy.random import randn
 from numpy import testing as npt
 
 from ...tests.helper import (raises, quantity_allclose as allclose,
@@ -116,19 +115,17 @@ def test_representations_api():
     c3 = PhysicsSphericalRepresentation(phi=120*u.deg, theta=85*u.deg, r=3*u.kpc)
 
     # first dimension must be length-3 if a lone `Quantity` is passed in.
-    c1 = CartesianRepresentation(np.random.randn(3, 100) * u.kpc)
+    c1 = CartesianRepresentation(randn(3, 100) * u.kpc)
     assert c1.xyz.shape[0] == 3
     assert c1.xyz.unit == u.kpc
     assert c1.x.shape[0] == 100
     assert c1.y.shape[0] == 100
     assert c1.z.shape[0] == 100
     # can also give each as separate keywords
-    CartesianRepresentation(x=np.random.randn(100)*u.kpc,
-                            y=np.random.randn(100)*u.kpc,
-                            z=np.random.randn(100)*u.kpc)
+    CartesianRepresentation(x=randn(100)*u.kpc, y=randn(100)*u.kpc, z=randn(100)*u.kpc)
     # if the units don't match but are all distances, they will automatically be
     # converted to match `x`
-    xarr, yarr, zarr = np.random.randn(3, 100)
+    xarr, yarr, zarr = randn(3, 100)
     c1 = CartesianRepresentation(x=xarr*u.kpc, y=yarr*u.kpc, z=zarr*u.kpc)
     c2 = CartesianRepresentation(x=xarr*u.kpc, y=yarr*u.kpc, z=zarr*u.pc)
     assert c1.xyz.unit == c2.xyz.unit == u.kpc
