@@ -1,19 +1,14 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-# TEST_UNICODE_LITERALS
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 """Test replacements for ERFA functions atciqz and aticq."""
 
 from itertools import product
-from ...tests.helper import (pytest,
-                             assert_quantity_allclose as assert_allclose)
 
+import pytest
+
+from ...tests.helper import assert_quantity_allclose as assert_allclose
 from ...time import Time
-
 from ... import _erfa as erfa
 from .utils import randomly_sample_sphere
 from ..builtin_frames.utils import get_jd12, atciqz, aticq
