@@ -1,9 +1,6 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import pytest
 import numpy as np
 
@@ -17,7 +14,6 @@ from ...tests.helper import (quantity_allclose as allclose,
 from .. import EarthLocation, CartesianRepresentation
 from ...time import Time
 
-from ...extern.six.moves import range
 
 # used below in the next parametrized test
 m31_sys = [ICRS, FK5, FK4, Galactic]
@@ -242,7 +238,6 @@ class TestHelioBaryCentric():
         self.obstime = Time("2013-02-02T23:00")
         self.wht_itrs = wht.get_itrs(obstime=self.obstime)
 
-    @pytest.mark.remote_data
     def test_heliocentric(self):
         gcrs = self.wht_itrs.transform_to(GCRS(obstime=self.obstime))
         helio = gcrs.transform_to(HCRS(obstime=self.obstime))
@@ -255,7 +250,6 @@ class TestHelioBaryCentric():
         assert np.sqrt(((helio.cartesian.xyz -
                          helio_slalib)**2).sum()) < 14. * u.km
 
-    @pytest.mark.remote_data
     def test_barycentric(self):
         gcrs = self.wht_itrs.transform_to(GCRS(obstime=self.obstime))
         bary = gcrs.transform_to(ICRS())
