@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import pytest
 import numpy as np
@@ -43,8 +41,8 @@ def test_faux_lsr(dt, symmetric):
                                      LSR2, ICRS, finite_difference_dt=dt,
                                      symmetric_finite_difference=symmetric)
     def lsr_to_icrs(lsr_coo, icrs_frame):
-        dt = lsr_coo.obstime - J2000
-        offset = lsr_coo.v_bary * dt.to(u.second)
+        dt = lsr_frame.obstime - J2000
+        offset = lsr_frame.v_bary * dt.to(u.second)
         return icrs_frame.realize_frame(lsr_coo.data - offset)
 
     ic = ICRS(ra=12.3*u.deg, dec=45.6*u.deg, distance=7.8*u.au,
@@ -142,7 +140,6 @@ def test_gcrs_diffs():
     assert np.all(np.abs(qtrisun2.data.differentials['s'].d_xyz) < 3e-5*u.km/u.s)
 
 
-@pytest.mark.remote_data
 def test_altaz_diffs():
     time = Time('J2015') + np.linspace(-1, 1, 1000)*u.day
     loc = get_builtin_sites()['greenwich']
@@ -175,21 +172,17 @@ _xfail = pytest.mark.xfail
 
 @pytest.mark.parametrize('distance', [1000*u.au,
                                       10*u.pc,
-                                      10*u.kpc,
-                                      100*u.kpc])
+                                      pytest.param(10*u.kpc, marks=_xfail),
+                                      pytest.param(100*u.kpc, marks=_xfail)])
+                                      # TODO:  make these not fail when the
+                                      # finite-difference numerical stability
+                                      # is improved
 def test_numerical_limits(distance):
     """
     Tests the numerical stability of the default settings for the finite
     difference transformation calculation.  This is *known* to fail for at
     >~1kpc, but this may be improved in future versions.
     """
-
-    if distance.unit == u.kpc:
-        # pytest.mark.parametrize syntax changed in pytest 3.1 to handle
-        # directly marking xfails, thus the workaround below to support
-        # pytest <3.1 for the 2.0.x LTS
-        pytest.xfail()
-
     time = Time('J2017') + np.linspace(-.5, .5, 100)*u.year
 
     icoo = ICRS(ra=0*u.deg, dec=10*u.deg, distance=distance,
