@@ -1,15 +1,11 @@
 # -*- coding: utf-8 -*-
 
-# TEST_UNICODE_LITERALS
 """
 Tests the Angle string formatting capabilities.  SkyCoord formatting is in
 test_sky_coord
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-from ...extern import six
 
 from ..angles import Angle
 from ... import units as u
@@ -112,7 +108,7 @@ def test_sexagesimal_rounding_up():
 
 def test_to_string_scalar():
     a = Angle(1.113355, unit=u.deg)
-    assert isinstance(a.to_string(), six.text_type)
+    assert isinstance(a.to_string(), str)
 
 
 def test_to_string_radian_with_precision():
