@@ -1,9 +1,6 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-import warnings
 
 import pytest
 import numpy as np
@@ -32,10 +29,7 @@ def fullstack_icrs():
 def fullstack_fiducial_altaz(fullstack_icrs):
     altazframe = AltAz(location=EarthLocation(lat=0*u.deg, lon=0*u.deg, height=0*u.m),
                        obstime=Time('J2000'))
-    with warnings.catch_warnings():  # Ignore remote_data warning
-        warnings.simplefilter('ignore')
-        result = fullstack_icrs.transform_to(altazframe)
-    return result
+    return fullstack_icrs.transform_to(altazframe)
 
 
 @pytest.fixture(scope="function", params=['J2000.1', 'J2010'])
@@ -49,12 +43,11 @@ def fullstack_locations(request):
                          height=request.param[0]*u.m)
 
 
-@pytest.fixture(scope="function",
-                params=[(0*u.bar, 0*u.deg_C, 0, 1*u.micron),
-                        (1*u.bar, 0*u.deg_C, 0*u.one, 1*u.micron),
-                        (1*u.bar, 10*u.deg_C, 0, 1*u.micron),
-                        (1*u.bar, 0*u.deg_C, 50*u.percent, 1*u.micron),
-                        (1*u.bar, 0*u.deg_C, 0, 21*u.cm)])
+@pytest.fixture(scope="function", params=[(0*u.bar, 0*u.deg_C, 0, 1*u.micron),
+                                        (1*u.bar, 0*u.deg_C, 0, 1*u.micron),
+                                        (1*u.bar, 10*u.deg_C, 0, 1*u.micron),
+                                        (1*u.bar, 0*u.deg_C, .5, 1*u.micron),
+                                        (1*u.bar, 0*u.deg_C, 0, 21*u.cm)])
 def fullstack_obsconditions(request):
     return request.param
 
@@ -75,7 +68,6 @@ def _erfa_check(ira, idec, astrom):
     return dct
 
 
-@pytest.mark.remote_data
 def test_iau_fullstack(fullstack_icrs, fullstack_fiducial_altaz,
                        fullstack_times, fullstack_locations,
                        fullstack_obsconditions):
@@ -124,23 +116,19 @@ def test_iau_fullstack(fullstack_icrs, fullstack_fiducial_altaz,
     lat = fullstack_locations.geodetic[1].to_value(u.radian)
     height = fullstack_locations.geodetic[2].to_value(u.m)
     jd1, jd2 = get_jd12(fullstack_times, 'utc')
-    pressure = fullstack_obsconditions[0].to_value(u.hPa)
-    temperature = fullstack_obsconditions[1].to_value(u.deg_C)
-    # Relative humidity can be a quantity or a number.
-    relative_humidity = u.Quantity(fullstack_obsconditions[2], u.one).value
-    obswl = fullstack_obsconditions[3].to_value(u.micron)
     astrom, eo = erfa.apco13(jd1, jd2,
                              fullstack_times.delta_ut1_utc,
                              lon, lat, height,
                              xp, yp,
-                             pressure, temperature, relative_humidity,
-                             obswl)
+                             fullstack_obsconditions[0].to_value(u.hPa),
+                             fullstack_obsconditions[1].to_value(u.deg_C),
+                             fullstack_obsconditions[2],
+                             fullstack_obsconditions[3].to_value(u.micron))
     erfadct = _erfa_check(fullstack_icrs.ra.rad, fullstack_icrs.dec.rad, astrom)
     npt.assert_allclose(erfadct['alt'], aacoo.alt.radian, atol=1e-7)
     npt.assert_allclose(erfadct['az'], aacoo.az.radian, atol=1e-7)
 
 
-@pytest.mark.remote_data
 def test_fiducial_roudtrip(fullstack_icrs, fullstack_fiducial_altaz):
     """
     Test the full transform from ICRS <-> AltAz
