@@ -1,13 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 from io import StringIO
 
+import pytest
 import numpy as np
 
 from .. import core, funcs
-from ...tests.helper import pytest, quantity_allclose as allclose
+from ...tests.helper import quantity_allclose as allclose
+from ...utils.compat import NUMPY_LT_1_14
 from ... import units as u
 
 try:
@@ -31,13 +31,16 @@ def test_init():
         h0bad = u.Quantity([70, 100], u.km / u.s / u.Mpc)
         cosmo = core.FlatLambdaCDM(H0=h0bad, Om0=0.27)
     with pytest.raises(ValueError):
-        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, m_nu=0.5)
+        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, Tcmb0=3, m_nu=0.5)
     with pytest.raises(ValueError):
         bad_mnu = u.Quantity([-0.3, 0.2, 0.1], u.eV)
-        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, m_nu=bad_mnu)
+        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, Tcmb0=3, m_nu=bad_mnu)
+    with pytest.raises(ValueError):
+        bad_mnu = u.Quantity([0.15, 0.2, 0.1], u.eV)
+        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, Tcmb0=3, Neff=2, m_nu=bad_mnu)
     with pytest.raises(ValueError):
         bad_mnu = u.Quantity([-0.3, 0.2], u.eV)  # 2, expecting 3
-        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, m_nu=bad_mnu)
+        cosmo = core.FlatLambdaCDM(H0=70, Om0=0.2, Tcmb0=3, m_nu=bad_mnu)
     with pytest.raises(ValueError):
         cosmo = core.FlatLambdaCDM(H0=70, Om0=0.27, Ob0=-0.04)
     with pytest.raises(ValueError):
@@ -48,6 +51,8 @@ def test_init():
     with pytest.raises(ValueError):
         cosmo = core.FlatLambdaCDM(H0=70, Om0=0.27)
         cosmo.Odm(1)
+    with pytest.raises(TypeError):
+        core.default_cosmology.validate(4)
 
 
 def test_basic():
@@ -100,9 +105,11 @@ def test_units():
 
     cosmo = core.FlatLambdaCDM(H0=70, Om0=0.27, Tcmb0=2.0)
     assert cosmo.comoving_distance(1.0).unit == u.Mpc
+    assert cosmo._comoving_distance_z1z2(1.0, 2.0).unit == u.Mpc
+    assert cosmo.comoving_transverse_distance(1.0).unit == u.Mpc
+    assert cosmo._comoving_transverse_distance_z1z2(1.0, 2.0).unit == u.Mpc
     assert cosmo.angular_diameter_distance(1.0).unit == u.Mpc
     assert cosmo.angular_diameter_distance_z1z2(1.0, 2.0).unit == u.Mpc
-    assert cosmo.comoving_distance(1.0).unit == u.Mpc
     assert cosmo.luminosity_distance(1.0).unit == u.Mpc
     assert cosmo.lookback_time(1.0).unit == u.Gyr
     assert cosmo.lookback_distance(1.0).unit == u.Mpc
@@ -232,7 +239,7 @@ def test_clone():
     assert allclose(newclone.Om(z), cmp.Om(z))
     assert allclose(newclone.H(z), cmp.H(z))
     assert allclose(newclone.luminosity_distance(z),
-                             cmp.luminosity_distance(z))
+                    cmp.luminosity_distance(z))
 
     # Now try changing multiple things
     newclone = cosmo.clone(name="New name", H0=65 * u.km / u.s / u.Mpc,
@@ -289,62 +296,82 @@ def test_clone():
         newclone = cosmo.clone(not_an_arg=4)
 
 
+def test_xtfuncs():
+    """ Test of absorption and lookback integrand"""
+    cosmo = core.LambdaCDM(70, 0.3, 0.5, Tcmb0=2.725)
+    z = np.array([2.0, 3.2])
+    assert allclose(cosmo.lookback_time_integrand(3), 0.052218976654969378,
+                    rtol=1e-4)
+    assert allclose(cosmo.lookback_time_integrand(z),
+                    [0.10333179, 0.04644541], rtol=1e-4)
+    assert allclose(cosmo.abs_distance_integrand(3), 3.3420145059180402,
+                    rtol=1e-4)
+    assert allclose(cosmo.abs_distance_integrand(z),
+                    [2.7899584, 3.44104758], rtol=1e-4)
+
+
 def test_repr():
     """ Test string representation of built in classes"""
-    cosmo = core.LambdaCDM(70, 0.3, 0.5)
-    expected = 'LambdaCDM(H0=70 km / (Mpc s), Om0=0.3, '\
-               'Ode0=0.5, Tcmb0=2.725 K, Neff=3.04, m_nu=[ 0.  0.  0.] eV, '\
-               'Ob0=None)'
+    cosmo = core.LambdaCDM(70, 0.3, 0.5, Tcmb0=2.725)
+    expected = ('LambdaCDM(H0=70 km / (Mpc s), Om0=0.3, '
+                'Ode0=0.5, Tcmb0=2.725 K, Neff=3.04, m_nu=[{}] eV, '
+                'Ob0=None)').format(' 0.  0.  0.' if NUMPY_LT_1_14 else
+                                    '0. 0. 0.')
     assert str(cosmo) == expected
 
-    cosmo = core.LambdaCDM(70, 0.3, 0.5, m_nu=u.Quantity(0.01, u.eV))
-    expected = 'LambdaCDM(H0=70 km / (Mpc s), Om0=0.3, Ode0=0.5, '\
-               'Tcmb0=2.725 K, Neff=3.04, m_nu=[ 0.01  0.01  0.01] eV, '\
-               'Ob0=None)'
+    cosmo = core.LambdaCDM(70, 0.3, 0.5, Tcmb0=2.725, m_nu=u.Quantity(0.01, u.eV))
+    expected = ('LambdaCDM(H0=70 km / (Mpc s), Om0=0.3, Ode0=0.5, '
+                'Tcmb0=2.725 K, Neff=3.04, m_nu=[{}] eV, '
+                'Ob0=None)').format(' 0.01  0.01  0.01' if NUMPY_LT_1_14 else
+                                    '0.01 0.01 0.01')
     assert str(cosmo) == expected
 
-    cosmo = core.FlatLambdaCDM(50.0, 0.27, Ob0=0.05)
-    expected = 'FlatLambdaCDM(H0=50 km / (Mpc s), Om0=0.27, '\
-               'Tcmb0=2.725 K, Neff=3.04, m_nu=[ 0.  0.  0.] eV, Ob0=0.05)'
+    cosmo = core.FlatLambdaCDM(50.0, 0.27, Tcmb0=3, Ob0=0.05)
+    expected = ('FlatLambdaCDM(H0=50 km / (Mpc s), Om0=0.27, '
+                'Tcmb0=3 K, Neff=3.04, m_nu=[{}] eV, Ob0=0.05)').format(
+                    ' 0.  0.  0.' if NUMPY_LT_1_14 else '0. 0. 0.')
     assert str(cosmo) == expected
 
-    cosmo = core.wCDM(60.0, 0.27, 0.6, w0=-0.8, name='test1')
-    expected = 'wCDM(name="test1", H0=60 km / (Mpc s), Om0=0.27, '\
-               'Ode0=0.6, w0=-0.8, Tcmb0=2.725 K, Neff=3.04, '\
-               'm_nu=[ 0.  0.  0.] eV, Ob0=None)'
+    cosmo = core.wCDM(60.0, 0.27, 0.6, Tcmb0=2.725, w0=-0.8, name='test1')
+    expected = ('wCDM(name="test1", H0=60 km / (Mpc s), Om0=0.27, '
+                'Ode0=0.6, w0=-0.8, Tcmb0=2.725 K, Neff=3.04, '
+                'm_nu=[{}] eV, Ob0=None)').format(
+                    ' 0.  0.  0.' if NUMPY_LT_1_14 else '0. 0. 0.')
     assert str(cosmo) == expected
 
     cosmo = core.FlatwCDM(65.0, 0.27, w0=-0.6, name='test2')
-    expected = 'FlatwCDM(name="test2", H0=65 km / (Mpc s), Om0=0.27, '\
-               'w0=-0.6, Tcmb0=2.725 K, Neff=3.04, m_nu=[ 0.  0.  0.] eV, '\
-               'Ob0=None)'
+    expected = ('FlatwCDM(name="test2", H0=65 km / (Mpc s), Om0=0.27, '
+                'w0=-0.6, Tcmb0=0 K, Neff=3.04, m_nu=None, Ob0=None)')
     assert str(cosmo) == expected
 
-    cosmo = core.w0waCDM(60.0, 0.25, 0.4, w0=-0.6, wa=0.1, name='test3')
-    expected = 'w0waCDM(name="test3", H0=60 km / (Mpc s), Om0=0.25, '\
-               'Ode0=0.4, w0=-0.6, wa=0.1, Tcmb0=2.725 K, Neff=3.04, '\
-               'm_nu=[ 0.  0.  0.] eV, Ob0=None)'
+    cosmo = core.w0waCDM(60.0, 0.25, 0.4, w0=-0.6, Tcmb0=2.725, wa=0.1, name='test3')
+    expected = ('w0waCDM(name="test3", H0=60 km / (Mpc s), Om0=0.25, '
+                'Ode0=0.4, w0=-0.6, wa=0.1, Tcmb0=2.725 K, Neff=3.04, '
+                'm_nu=[{}] eV, Ob0=None)').format(
+                    ' 0.  0.  0.' if NUMPY_LT_1_14 else '0. 0. 0.')
     assert str(cosmo) == expected
 
     cosmo = core.Flatw0waCDM(55.0, 0.35, w0=-0.9, wa=-0.2, name='test4',
                              Ob0=0.0456789)
-    expected = 'Flatw0waCDM(name="test4", H0=55 km / (Mpc s), Om0=0.35, '\
-               'w0=-0.9, Tcmb0=2.725 K, Neff=3.04, m_nu=[ 0.  0.  0.] eV, '\
-               'Ob0=0.0457)'
+    expected = ('Flatw0waCDM(name="test4", H0=55 km / (Mpc s), Om0=0.35, '
+                'w0=-0.9, Tcmb0=0 K, Neff=3.04, m_nu=None, '
+                'Ob0=0.0457)')
     assert str(cosmo) == expected
 
     cosmo = core.wpwaCDM(50.0, 0.3, 0.3, wp=-0.9, wa=-0.2,
                          zp=0.3, name='test5')
-    expected = 'wpwaCDM(name="test5", H0=50 km / (Mpc s), Om0=0.3, '\
-               'Ode0=0.3, wp=-0.9, wa=-0.2, zp=0.3, Tcmb0=2.725 K, '\
-               'Neff=3.04, m_nu=[ 0.  0.  0.] eV, Ob0=None)'
+    expected = ('wpwaCDM(name="test5", H0=50 km / (Mpc s), Om0=0.3, '
+                'Ode0=0.3, wp=-0.9, wa=-0.2, zp=0.3, Tcmb0=0 K, '
+                'Neff=3.04, m_nu=None, Ob0=None)')
     assert str(cosmo) == expected
 
-    cosmo = core.w0wzCDM(55.0, 0.4, 0.8, w0=-1.05, wz=-0.2,
+    cosmo = core.w0wzCDM(55.0, 0.4, 0.8, w0=-1.05, wz=-0.2, Tcmb0=2.725,
                          m_nu=u.Quantity([0.001, 0.01, 0.015], u.eV))
-    expected = 'w0wzCDM(H0=55 km / (Mpc s), Om0=0.4, Ode0=0.8, w0=-1.05, '\
-               'wz=-0.2 Tcmb0=2.725 K, Neff=3.04, '\
-               'm_nu=[ 0.001  0.01   0.015] eV, Ob0=None)'
+    expected = ('w0wzCDM(H0=55 km / (Mpc s), Om0=0.4, Ode0=0.8, w0=-1.05, '
+                'wz=-0.2 Tcmb0=2.725 K, Neff=3.04, '
+                'm_nu=[{}] eV, Ob0=None)').format(
+                    ' 0.001  0.01   0.015' if NUMPY_LT_1_14 else
+                    '0.001 0.01  0.015')
     assert str(cosmo) == expected
 
 
@@ -366,15 +393,15 @@ def test_flat_z1():
 
     # The order of values below is Wright, Kempner, iCosmos'
     assert allclose(cosmo.comoving_distance(z),
-                             [3364.5, 3364.8, 3364.7988] * u.Mpc, rtol=1e-4)
+                    [3364.5, 3364.8, 3364.7988] * u.Mpc, rtol=1e-4)
     assert allclose(cosmo.angular_diameter_distance(z),
-                             [1682.3, 1682.4, 1682.3994] * u.Mpc, rtol=1e-4)
+                    [1682.3, 1682.4, 1682.3994] * u.Mpc, rtol=1e-4)
     assert allclose(cosmo.luminosity_distance(z),
-                             [6729.2, 6729.6, 6729.5976] * u.Mpc, rtol=1e-4)
+                    [6729.2, 6729.6, 6729.5976] * u.Mpc, rtol=1e-4)
     assert allclose(cosmo.lookback_time(z),
-                             [7.841, 7.84178, 7.843] * u.Gyr, rtol=1e-3)
+                    [7.841, 7.84178, 7.843] * u.Gyr, rtol=1e-3)
     assert allclose(cosmo.lookback_distance(z),
-                             [2404.0, 2404.24, 2404.4] * u.Mpc, rtol=1e-3)
+                    [2404.0, 2404.24, 2404.4] * u.Mpc, rtol=1e-3)
 
 
 def test_zeroing():
@@ -382,9 +409,13 @@ def test_zeroing():
     # Make sure Ode = 0 behaves that way
     cosmo = core.LambdaCDM(H0=70, Om0=0.27, Ode0=0.0)
     assert allclose(cosmo.Ode([0, 1, 2, 3]), [0, 0, 0, 0])
-    # Ogamma0
+    assert allclose(cosmo.Ode(1), 0)
+    # Ogamma0 and Onu
     cosmo = core.FlatLambdaCDM(H0=70, Om0=0.27, Tcmb0=0.0)
+    assert allclose(cosmo.Ogamma(1.5), [0, 0, 0, 0])
     assert allclose(cosmo.Ogamma([0, 1, 2, 3]), [0, 0, 0, 0])
+    assert allclose(cosmo.Onu(1.5), [0, 0, 0, 0])
+    assert allclose(cosmo.Onu([0, 1, 2, 3]), [0, 0, 0, 0])
     # Obaryon
     cosmo = core.LambdaCDM(H0=70, Om0=0.27, Ode0=0.73, Ob0=0.0)
     assert allclose(cosmo.Ob([0, 1, 2, 3]), [0, 0, 0, 0])
@@ -393,7 +424,6 @@ def test_zeroing():
 # This class is to test whether the routines work correctly
 # if one only overloads w(z)
 class test_cos_sub(core.FLRW):
-
     def __init__(self):
         core.FLRW.__init__(self, 70.0, 0.27, 0.73, Tcmb0=0.0,
                            name="test_cos")
@@ -402,6 +432,18 @@ class test_cos_sub(core.FLRW):
     def w(self, z):
         return self._w0 * np.ones_like(z)
 
+# Similar, but with neutrinos
+
+
+class test_cos_subnu(core.FLRW):
+    def __init__(self):
+        core.FLRW.__init__(self, 70.0, 0.27, 0.73, Tcmb0=3.0,
+                           m_nu=0.1 * u.eV, name="test_cos_nu")
+        self._w0 = -0.8
+
+    def w(self, z):
+        return self._w0 * np.ones_like(z)
+
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_de_subclass():
@@ -426,6 +468,8 @@ def test_de_subclass():
     assert allclose(cosmo.de_density_scale([0.5, 1.0]),
                     [1.12934694, 1.23114444], rtol=1e-4)
 
+    # Add neutrinos for efunc, inv_efunc
+
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_varyde_lumdist_mathematica():
@@ -479,11 +523,11 @@ def test_matter():
     assert allclose(tcos.Om(0), 0.3)
     assert allclose(tcos.Ob(0), 0.045)
     z = np.array([0.0, 0.5, 1.0, 2.0])
-    assert allclose(tcos.Om(z), [0.3, 0.59112134, 0.77387435, 0.91974179],
+    assert allclose(tcos.Om(z), [0.3, 0.59124088, 0.77419355, 0.92045455],
                     rtol=1e-4)
     assert allclose(tcos.Ob(z),
-                    [0.045, 0.08866820, 0.11608115, 0.13796127], rtol=1e-4)
-    assert allclose(tcos.Odm(z), [0.255, 0.50245314, 0.6577932, 0.78178052],
+                    [0.045, 0.08868613, 0.11612903, 0.13806818], rtol=1e-4)
+    assert allclose(tcos.Odm(z), [0.255, 0.50255474, 0.65806452, 0.78238636],
                     rtol=1e-4)
     # Consistency of dark and baryonic matter evolution with all
     # non-relativistic matter
@@ -579,7 +623,7 @@ def test_ogamma():
     assert allclose(cosmo.comoving_distance(z), targvals, rtol=1e-5)
 
     # And integers for z
-    assert allclose(cosmo.comoving_distance(z.astype(np.int)),
+    assert allclose(cosmo.comoving_distance(z.astype(int)),
                     targvals, rtol=1e-5)
 
     # Try Tcmb0 = 4
@@ -620,7 +664,12 @@ def test_tnu():
 
 
 def test_efunc_vs_invefunc():
-    # Test that efunc and inv_efunc give the same values
+    """ Test that efunc and inv_efunc give inverse values"""
+
+    # Note that all of the subclasses here don't need
+    #  scipy because they don't need to call de_density_scale
+    # The test following this tests the case where that is needed.
+
     z0 = 0.5
     z = np.array([0.5, 1.0, 2.0, 5.0])
 
@@ -658,14 +707,20 @@ def test_efunc_vs_invefunc():
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_efunc_vs_invefunc_flrw():
+    """ Test that efunc and inv_efunc give inverse values"""
     z0 = 0.5
     z = np.array([0.5, 1.0, 2.0, 5.0])
 
     # FLRW is abstract, so requires test_cos_sub defined earlier
-    # This requires scipy, unlike the built-ins
+    # This requires scipy, unlike the built-ins, because it
+    # calls de_density_scale, which has an integral in it
     cosmo = test_cos_sub()
     assert allclose(cosmo.efunc(z0), 1.0 / cosmo.inv_efunc(z0))
     assert allclose(cosmo.efunc(z), 1.0 / cosmo.inv_efunc(z))
+    # Add neutrinos
+    cosmo = test_cos_subnu()
+    assert allclose(cosmo.efunc(z0), 1.0 / cosmo.inv_efunc(z0))
+    assert allclose(cosmo.efunc(z), 1.0 / cosmo.inv_efunc(z))
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -897,34 +952,39 @@ def test_integral():
 
 def test_wz():
     cosmo = core.LambdaCDM(H0=70, Om0=0.3, Ode0=0.70)
+    assert allclose(cosmo.w(1.0), -1.)
     assert allclose(cosmo.w([0.1, 0.2, 0.5, 1.5, 2.5, 11.5]),
                     [-1., -1, -1, -1, -1, -1])
 
     cosmo = core.wCDM(H0=70, Om0=0.3, Ode0=0.70, w0=-0.5)
+    assert allclose(cosmo.w(1.0), -0.5)
     assert allclose(cosmo.w([0.1, 0.2, 0.5, 1.5, 2.5, 11.5]),
                     [-0.5, -0.5, -0.5, -0.5, -0.5, -0.5])
     assert allclose(cosmo.w0, -0.5)
 
     cosmo = core.w0wzCDM(H0=70, Om0=0.3, Ode0=0.70, w0=-1, wz=0.5)
+    assert allclose(cosmo.w(1.0), -0.5)
     assert allclose(cosmo.w([0.0, 0.5, 1.0, 1.5, 2.3]),
-                       [-1.0, -0.75, -0.5, -0.25, 0.15])
+                    [-1.0, -0.75, -0.5, -0.25, 0.15])
     assert allclose(cosmo.w0, -1.0)
     assert allclose(cosmo.wz, 0.5)
 
     cosmo = core.w0waCDM(H0=70, Om0=0.3, Ode0=0.70, w0=-1, wa=-0.5)
     assert allclose(cosmo.w0, -1.0)
     assert allclose(cosmo.wa, -0.5)
+    assert allclose(cosmo.w(1.0), -1.25)
     assert allclose(cosmo.w([0.0, 0.5, 1.0, 1.5, 2.3]),
-                       [-1, -1.16666667, -1.25, -1.3, -1.34848485])
+                    [-1, -1.16666667, -1.25, -1.3, -1.34848485])
 
     cosmo = core.wpwaCDM(H0=70, Om0=0.3, Ode0=0.70, wp=-0.9,
                          wa=0.2, zp=0.5)
     assert allclose(cosmo.wp, -0.9)
     assert allclose(cosmo.wa, 0.2)
     assert allclose(cosmo.zp, 0.5)
+    assert allclose(cosmo.w(0.5), -0.9)
     assert allclose(cosmo.w([0.1, 0.2, 0.5, 1.5, 2.5, 11.5]),
-                       [-0.94848485, -0.93333333, -0.9, -0.84666667,
-                        -0.82380952, -0.78266667])
+                    [-0.94848485, -0.93333333, -0.9, -0.84666667,
+                     -0.82380952, -0.78266667])
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -982,10 +1042,22 @@ def test_age():
     # WMAP7 but with Omega_relativisitic = 0
     tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=0.0)
     assert allclose(tcos.hubble_time, 13.889094057856937 * u.Gyr)
+    assert allclose(tcos.age(4), 1.5823603508870991 * u.Gyr)
     assert allclose(tcos.age([1., 5.]),
                     [5.97113193, 1.20553129] * u.Gyr)
     assert allclose(tcos.age([1, 5]), [5.97113193, 1.20553129] * u.Gyr)
 
+    # Add relativistic species
+    tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=3.0)
+    assert allclose(tcos.age(4), 1.5773003779230699 * u.Gyr)
+    assert allclose(tcos.age([1, 5]), [5.96344942, 1.20093077] * u.Gyr)
+
+    # And massive neutrinos
+    tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=3.0,
+                              m_nu=0.1 * u.eV)
+    assert allclose(tcos.age(4), 1.5546485439853412 * u.Gyr)
+    assert allclose(tcos.age([1, 5]), [5.88448152, 1.18383759] * u.Gyr)
+
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_distmod():
@@ -1016,45 +1088,120 @@ def test_critical_density():
     #  units by default; see the comment at the top of core.py
     tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=0.0)
     assert allclose(tcos.critical_density0,
-                    9.31000324385361e-30 * u.g / u.cm**3)
+                    9.309668456020899e-30 * u.g / u.cm**3)
     assert allclose(tcos.critical_density0,
                     tcos.critical_density(0))
     assert allclose(tcos.critical_density([1, 5]),
-                    [2.70362491e-29, 5.53758986e-28] * u.g / u.cm**3)
+                    [2.70352772e-29, 5.53739080e-28] * u.g / u.cm**3)
     assert allclose(tcos.critical_density([1., 5.]),
-                    [2.70362491e-29, 5.53758986e-28] * u.g / u.cm**3)
+                    [2.70352772e-29, 5.53739080e-28] * u.g / u.cm**3)
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_comoving_distance_z1z2():
+    tcos = core.LambdaCDM(100, 0.3, 0.8, Tcmb0=0.0)
+    with pytest.raises(ValueError):  # test diff size z1, z2 fail
+        tcos._comoving_distance_z1z2((1, 2), (3, 4, 5))
+    # Comoving distances are invertible
+    assert allclose(tcos._comoving_distance_z1z2(1, 2),
+                    -tcos._comoving_distance_z1z2(2, 1))
+
+    z1 = 0, 0, 2, 0.5, 1
+    z2 = 2, 1, 1, 2.5, 1.1
+    results = (3767.90579253,
+               2386.25591391,
+               -1381.64987862,
+               2893.11776663,
+               174.1524683) * u.Mpc
+
+    assert allclose(tcos._comoving_distance_z1z2(z1, z2),
+                    results)
+
+
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_comoving_transverse_distance_z1z2():
+    tcos = core.FlatLambdaCDM(100, 0.3, Tcmb0=0.0)
+    with pytest.raises(ValueError):  # test diff size z1, z2 fail
+        tcos._comoving_transverse_distance_z1z2((1, 2), (3, 4, 5))
+    # Tests that should actually work, target values computed with
+    # http://www.astro.multivax.de:8000/phillip/angsiz_prog/README.HTML
+    # Kayser, Helbig, and Schramm (Astron.Astrophys. 318 (1997) 680-686)
+    assert allclose(tcos._comoving_transverse_distance_z1z2(1, 2),
+                    1313.2232194828466 * u.Mpc)
+
+    # In a flat universe comoving distance and comoving transverse
+    # distance are identical
+    z1 = 0, 0, 2, 0.5, 1
+    z2 = 2, 1, 1, 2.5, 1.1
+
+    assert allclose(tcos._comoving_distance_z1z2(z1, z2),
+                    tcos._comoving_transverse_distance_z1z2(z1, z2))
+
+    # Test non-flat cases to avoid simply testing
+    # comoving_distance_z1z2. Test array, array case.
+    tcos = core.LambdaCDM(100, 0.3, 0.5, Tcmb0=0.0)
+    results = (3535.931375645655,
+               2226.430046551708,
+               -1208.6817970036532,
+               2595.567367601969,
+               151.36592003406884) * u.Mpc
+
+    assert allclose(tcos._comoving_transverse_distance_z1z2(z1, z2),
+                    results)
+
+    # Test positive curvature with scalar, array combination.
+    tcos = core.LambdaCDM(100, 1.0, 0.2, Tcmb0=0.0)
+    z1 = 0.1
+    z2 = 0, 0.1, 0.2, 0.5, 1.1, 2
+    results = (-281.31602666724865,
+               0.,
+               248.58093707820436,
+               843.9331377460543,
+               1618.6104987686672,
+               2287.5626543279927) * u.Mpc
+
+    assert allclose(tcos._comoving_transverse_distance_z1z2(z1, z2),
+                    results)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_angular_diameter_distance_z1z2():
-
-    with pytest.raises(core.CosmologyError):  # test neg Ok fail
-        tcos = core.LambdaCDM(H0=70.4, Om0=0.272, Ode0=0.8, Tcmb0=0.0)
-        tcos.angular_diameter_distance_z1z2(1, 2)
-
     tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=0.0)
     with pytest.raises(ValueError):  # test diff size z1, z2 fail
         tcos.angular_diameter_distance_z1z2([1, 2], [3, 4, 5])
-    with pytest.raises(ValueError):  # test z1 > z2 fail
-        tcos.angular_diameter_distance_z1z2(4, 3)
     # Tests that should actually work
     assert allclose(tcos.angular_diameter_distance_z1z2(1, 2),
-                             646.22968662822018 * u.Mpc)
-    z1 = 0, 0, 1, 0.5, 1
-    z2 = 2, 1, 2, 2.5, 1.1
+                    646.22968662822018 * u.Mpc)
+
+    z1 = 0, 0, 2, 0.5, 1
+    z2 = 2, 1, 1, 2.5, 1.1
     results = (1760.0628637762106,
                1670.7497657219858,
-               646.22968662822018,
+               -969.34452994,
                1159.0970895962193,
                115.72768186186921) * u.Mpc
 
     assert allclose(tcos.angular_diameter_distance_z1z2(z1, z2),
                     results)
 
-    # Non-flat (positive Ocurv) test
+    z1 = 0.1
+    z2 = 0.1, 0.2, 0.5, 1.1, 2
+    results = (0.,
+               332.09893173,
+               986.35635069,
+               1508.37010062,
+               1621.07937976) * u.Mpc
+    assert allclose(tcos.angular_diameter_distance_z1z2(0.1, z2),
+                    results)
+
+    # Non-flat (positive Ok0) test
     tcos = core.LambdaCDM(H0=70.4, Om0=0.2, Ode0=0.5, Tcmb0=0.0)
     assert allclose(tcos.angular_diameter_distance_z1z2(1, 2),
                     620.1175337852428 * u.Mpc)
+    # Non-flat (negative Ok0) test
+    tcos = core.LambdaCDM(H0=100, Om0=2, Ode0=1, Tcmb0=0.0)
+    assert allclose(tcos.angular_diameter_distance_z1z2(1, 2),
+                    228.42914659246014 * u.Mpc)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -1072,7 +1219,7 @@ def test_absorption_distance():
 def test_massivenu_basic():
     # Test no neutrinos case
     tcos = core.FlatLambdaCDM(70.4, 0.272, Neff=4.05,
-                              m_nu=u.Quantity(0, u.eV))
+                              Tcmb0=2.725 * u.K, m_nu=u.Quantity(0, u.eV))
     assert allclose(tcos.Neff, 4.05)
     assert not tcos.has_massive_nu
     mnu = tcos.m_nu
@@ -1084,8 +1231,14 @@ def test_massivenu_basic():
     assert allclose(tcos.nu_relative_density(1), 0.22710731766 * 4.05,
                     rtol=1e-6)
 
+    # Alternative no neutrinos case
+    tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=0 * u.K,
+                              m_nu=u.Quantity(0.4, u.eV))
+    assert not tcos.has_massive_nu
+    assert tcos.m_nu is None
+
     # Test basic setting, retrieval of values
-    tcos = core.FlatLambdaCDM(70.4, 0.272,
+    tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=2.725 * u.K,
                               m_nu=u.Quantity([0.0, 0.01, 0.02], u.eV))
     assert tcos.has_massive_nu
     mnu = tcos.m_nu
@@ -1094,8 +1247,8 @@ def test_massivenu_basic():
     assert allclose(mnu, [0.0, 0.01, 0.02] * u.eV)
 
     # All massive neutrinos case
-    tcos = core.FlatLambdaCDM(70.4, 0.272, m_nu=u.Quantity(0.1, u.eV),
-                              Neff=3.1)
+    tcos = core.FlatLambdaCDM(70.4, 0.272, Tcmb0=2.725,
+                              m_nu=u.Quantity(0.1, u.eV), Neff=3.1)
     assert allclose(tcos.Neff, 3.1)
     assert tcos.has_massive_nu
     mnu = tcos.m_nu
@@ -1104,6 +1257,164 @@ def test_massivenu_basic():
     assert allclose(mnu, [0.1, 0.1, 0.1] * u.eV)
 
 
+@pytest.mark.skipif('not HAS_SCIPY')
+def test_distances():
+    # Test distance calculations for various special case
+    #  scenarios (no relativistic species, normal, massive neutrinos)
+    # These do not come from external codes -- they are just internal
+    #  checks to make sure nothing changes if we muck with the distance
+    #  calculators
+
+    z = np.array([1.0, 2.0, 3.0, 4.0])
+
+    # The pattern here is: no relativistic species, the relativistic
+    # species with massless neutrinos, then massive neutrinos
+    cos = core.LambdaCDM(75.0, 0.25, 0.5, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [2953.93001902, 4616.7134253, 5685.07765971,
+                     6440.80611897] * u.Mpc, rtol=1e-4)
+    cos = core.LambdaCDM(75.0, 0.25, 0.6, Tcmb0=3.0, Neff=3,
+                         m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [3037.12620424, 4776.86236327, 5889.55164479,
+                     6671.85418235] * u.Mpc, rtol=1e-4)
+    cos = core.LambdaCDM(75.0, 0.3, 0.4, Tcmb0=3.0, Neff=3,
+                         m_nu=u.Quantity(10.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2471.80626824, 3567.1902565, 4207.15995626,
+                     4638.20476018] * u.Mpc, rtol=1e-4)
+    # Flat
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [3180.83488552, 5060.82054204, 6253.6721173,
+                     7083.5374303] * u.Mpc, rtol=1e-4)
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=3.0, Neff=3,
+                              m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [3180.42662867, 5059.60529655, 6251.62766102,
+                     7080.71698117] * u.Mpc, rtol=1e-4)
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=3.0, Neff=3,
+                              m_nu=u.Quantity(10.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2337.54183142, 3371.91131264, 3988.40711188,
+                     4409.09346922] * u.Mpc, rtol=1e-4)
+    # Add w
+    cos = core.FlatwCDM(75.0, 0.25, w0=-1.05, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [3216.8296894, 5117.2097601, 6317.05995437,
+                     7149.68648536] * u.Mpc, rtol=1e-4)
+    cos = core.FlatwCDM(75.0, 0.25, w0=-0.95, Tcmb0=3.0, Neff=3,
+                    m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [3143.56537758, 5000.32196494, 6184.11444601,
+                     7009.80166062] * u.Mpc, rtol=1e-4)
+    cos = core.FlatwCDM(75.0, 0.25, w0=-0.9, Tcmb0=3.0, Neff=3,
+                    m_nu=u.Quantity(10.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2337.76035371, 3372.1971387, 3988.71362289,
+                     4409.40817174] * u.Mpc, rtol=1e-4)
+    # Non-flat w
+    cos = core.wCDM(75.0, 0.25, 0.4, w0=-0.9, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [2849.6163356, 4428.71661565, 5450.97862778,
+                     6179.37072324] * u.Mpc, rtol=1e-4)
+    cos = core.wCDM(75.0, 0.25, 0.4, w0=-1.1, Tcmb0=3.0, Neff=3,
+                    m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2904.35580229, 4511.11471267, 5543.43643353,
+                     6275.9206788] * u.Mpc, rtol=1e-4)
+    cos = core.wCDM(75.0, 0.25, 0.4, w0=-0.9, Tcmb0=3.0, Neff=3,
+                    m_nu=u.Quantity(10.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2473.32522734, 3581.54519631, 4232.41674426,
+                     4671.83818117] * u.Mpc, rtol=1e-4)
+    # w0wa
+    cos = core.w0waCDM(75.0, 0.3, 0.6, w0=-0.9, wa=0.1, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [2937.7807638, 4572.59950903, 5611.52821924,
+                     6339.8549956] * u.Mpc, rtol=1e-4)
+    cos = core.w0waCDM(75.0, 0.25, 0.5, w0=-0.9, wa=0.1, Tcmb0=3.0, Neff=3,
+                       m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2907.34722624, 4539.01723198, 5593.51611281,
+                     6342.3228444] * u.Mpc, rtol=1e-4)
+    cos = core.w0waCDM(75.0, 0.25, 0.5, w0=-0.9, wa=0.1, Tcmb0=3.0, Neff=3,
+                       m_nu=u.Quantity(10.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2507.18336722, 3633.33231695, 4292.44746919,
+                     4736.35404638] * u.Mpc, rtol=1e-4)
+    # Flatw0wa
+    cos = core.Flatw0waCDM(75.0, 0.25, w0=-0.95, wa=0.15, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [3123.29892781, 4956.15204302, 6128.15563818,
+                     6948.26480378] * u.Mpc, rtol=1e-4)
+    cos = core.Flatw0waCDM(75.0, 0.25, w0=-0.95, wa=0.15, Tcmb0=3.0, Neff=3,
+                           m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [3122.92671907, 4955.03768936, 6126.25719576,
+                     6945.61856513] * u.Mpc, rtol=1e-4)
+    cos = core.Flatw0waCDM(75.0, 0.25, w0=-0.95, wa=0.15, Tcmb0=3.0, Neff=3,
+                           m_nu=u.Quantity(10.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2337.70072701, 3372.13719963, 3988.6571093,
+                     4409.35399673] * u.Mpc, rtol=1e-4)
+    # wpwa
+    cos = core.wpwaCDM(75.0, 0.3, 0.6, wp=-0.9, zp=0.5, wa=0.1, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [2954.68975298, 4599.83254834, 5643.04013201,
+                     6373.36147627] * u.Mpc, rtol=1e-4)
+    cos = core.wpwaCDM(75.0, 0.25, 0.5, wp=-0.9, zp=0.4, wa=0.1,
+                       Tcmb0=3.0, Neff=3, m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2919.00656215, 4558.0218123, 5615.73412391,
+                     6366.10224229] * u.Mpc, rtol=1e-4)
+    cos = core.wpwaCDM(75.0, 0.25, 0.5, wp=-0.9, zp=1.0, wa=0.1, Tcmb0=3.0,
+                       Neff=4, m_nu=u.Quantity(5.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2629.48489827, 3874.13392319, 4614.31562397,
+                     5116.51184842] * u.Mpc, rtol=1e-4)
+
+    # w0wz
+    cos = core.w0wzCDM(75.0, 0.3, 0.6, w0=-0.9, wz=0.1, Tcmb0=0.0)
+    assert allclose(cos.comoving_distance(z),
+                    [3051.68786716, 4756.17714818, 5822.38084257,
+                     6562.70873734] * u.Mpc, rtol=1e-4)
+    cos = core.w0wzCDM(75.0, 0.25, 0.5, w0=-0.9, wz=0.1,
+                       Tcmb0=3.0, Neff=3, m_nu=u.Quantity(0.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2997.8115653, 4686.45599916, 5764.54388557,
+                     6524.17408738] * u.Mpc, rtol=1e-4)
+    cos = core.w0wzCDM(75.0, 0.25, 0.5, w0=-0.9, wz=0.1, Tcmb0=3.0,
+                       Neff=4, m_nu=u.Quantity(5.0, u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2676.73467639, 3940.57967585, 4686.90810278,
+                     5191.54178243] * u.Mpc, rtol=1e-4)
+
+    # Also test different numbers of massive neutrinos
+    # for FlatLambdaCDM to give the scalar nu density functions a
+    # work out
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=3.0,
+                             m_nu=u.Quantity([10.0, 0, 0], u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2777.71589173, 4186.91111666, 5046.0300719,
+                     5636.10397302] * u.Mpc, rtol=1e-4)
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=3.0,
+                             m_nu=u.Quantity([10.0, 5, 0], u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2636.48149391, 3913.14102091, 4684.59108974,
+                     5213.07557084] * u.Mpc, rtol=1e-4)
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=3.0,
+                             m_nu=u.Quantity([4.0, 5, 9], u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2563.5093049, 3776.63362071, 4506.83448243,
+                     5006.50158829] * u.Mpc, rtol=1e-4)
+    cos = core.FlatLambdaCDM(75.0, 0.25, Tcmb0=3.0, Neff=4.2,
+                             m_nu=u.Quantity([1.0, 4.0, 5, 9], u.eV))
+    assert allclose(cos.comoving_distance(z),
+                    [2525.58017482, 3706.87633298, 4416.58398847,
+                     4901.96669755] * u.Mpc, rtol=1e-4)
+
+
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_massivenu_density():
     # Testing neutrino density calculation
@@ -1168,7 +1479,7 @@ def test_massivenu_density():
     assert allclose(tcos.Onu(ztest), onu_exp, rtol=5e-3)
 
     # Integer redshifts
-    ztest = ztest.astype(np.int)
+    ztest = ztest.astype(int)
     assert allclose(tcos.nu_relative_density(ztest), nurel_exp,
                     rtol=5e-3)
     assert allclose(tcos.Onu(ztest), onu_exp, rtol=5e-3)
@@ -1216,12 +1527,14 @@ def test_z_at_value_roundtrip():
     """
     z = 0.5
 
-    # Skip Ok, w, de_density_scale because in the Planck13 cosmolgy
+    # Skip Ok, w, de_density_scale because in the Planck13 cosmology
     # they are redshift independent and hence uninvertable,
-    # angular_diameter_distance_z1z2 takes multiple arguments, so requires
-    #  special handling
+    # *_distance_z1z2 methods take multiple arguments, so require
+    # special handling
     # clone isn't a redshift-dependent method
-    skip = ('Ok', 'angular_diameter_distance_z1z2', 'clone',
+    skip = ('Ok',
+            'angular_diameter_distance_z1z2',
+            'clone',
             'de_density_scale', 'w')
 
     import inspect
@@ -1239,9 +1552,14 @@ def test_z_at_value_roundtrip():
         assert allclose(z, funcs.z_at_value(func, fval, zmax=1.5),
                         rtol=2e-8)
 
-    # Test angular_diameter_distance_z1z2
+    # Test distance functions between two redshifts
     z2 = 2.0
-    func = lambda z1: core.Planck13.angular_diameter_distance_z1z2(z1, z2)
-    fval = func(z)
-    assert allclose(z, funcs.z_at_value(func, fval, zmax=1.5),
-                    rtol=2e-8)
+    func_z1z2 = [lambda z1: core.Planck13._comoving_distance_z1z2(z1, z2),
+                 lambda z1:
+                 core.Planck13._comoving_transverse_distance_z1z2(z1, z2),
+                 lambda z1:
+                 core.Planck13.angular_diameter_distance_z1z2(z1, z2)]
+    for func in func_z1z2:
+        fval = func(z)
+        assert allclose(z, funcs.z_at_value(func, fval, zmax=1.5),
+                        rtol=2e-8)
