@@ -9,12 +9,10 @@ basic.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-from __future__ import absolute_import, division, print_function
 
 import re
 
 from . import core
-from ...extern.six.moves import zip
 
 
 class BasicHeader(core.BaseHeader):
@@ -137,12 +135,7 @@ class CommentedHeader(Basic):
     the header comment character. ``header_start`` can be used to specify the
     line index of column names, and it can be a negative index (for example -1
     for the last commented line).  The default delimiter is the <space>
-    character.
-
-    This matches the format produced by ``np.savetxt()``, with ``delimiter=','``,
-    and ``header='<comma-delimited-column-names-list>'``.
-
-    Example::
+    character.::
 
       # col1 col2 col3
       # Comment line
@@ -160,15 +153,12 @@ class CommentedHeader(Basic):
         Read input data (file-like object, filename, list of strings, or
         single string) into a Table and return the result.
         """
-        out = super(CommentedHeader, self).read(table)
+        out = super().read(table)
 
-        # Strip off the comment line set as the header line for
-        # commented_header format (first by default).
+        # Strip off first comment since this is the header line for
+        # commented_header format.
         if 'comments' in out.meta:
-            idx = self.header.start_line
-            if idx < 0:
-                idx = len(out.meta['comments']) + idx
-            out.meta['comments'] = out.meta['comments'][:idx] + out.meta['comments'][idx+1:]
+            out.meta['comments'] = out.meta['comments'][1:]
             if not out.meta['comments']:
                 del out.meta['comments']
 
