@@ -8,25 +8,25 @@ core.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-from __future__ import absolute_import, division, print_function
 
+import copy
+import csv
+import functools
+import itertools
+import operator
 import os
 import re
-import csv
-import itertools
-import functools
-import numpy
 import warnings
-import copy
-from collections import OrderedDict
 
-from ...extern import six
-from ...extern.six.moves import zip
-from ...extern.six.moves import cStringIO as StringIO
+from collections import OrderedDict
+from contextlib import suppress
+from io import StringIO
+
+import numpy
+
 from ...utils.exceptions import AstropyWarning
 
 from ...table import Table
-from ...utils.compat import ignored
 from ...utils.data import get_readable_fileobj
 from . import connect
 
@@ -37,7 +37,7 @@ FORMAT_CLASSES = {}
 FAST_CLASSES = {}
 
 
-class CsvWriter(object):
+class CsvWriter:
     """
     Internal class to replace the csv writer ``writerow`` and ``writerows``
     functions so that in the case of ``delimiter=' '`` and
@@ -138,17 +138,20 @@ class MaskedConstant(numpy.ma.core.MaskedConstant):
     """A trivial extension of numpy.ma.masked
 
     We want to be able to put the generic term ``masked`` into a dictionary.
-    In python 2.7 we can just use ``numpy.ma.masked``, but in python 3.1 and 3.2 that
-    is not hashable, see https://github.com/numpy/numpy/issues/4660
-    So, we need to extend it here with a hash value.
+    The constant ``numpy.ma.masked`` is not hashable (see
+    https://github.com/numpy/numpy/issues/4660), so we need to extend it
+    here with a hash value.
     """
+
     def __hash__(self):
         '''All instances of this class shall have the same hash.'''
         # Any large number will do.
         return 1234567890
 
+
 masked = MaskedConstant()
 
+
 class InconsistentTableError(ValueError):
     """
     Indicates that an input table is inconsistent in some way.
@@ -157,6 +160,7 @@ class InconsistentTableError(ValueError):
     this class if a data row doesn't match the header.
     """
 
+
 class OptionalTableImportError(ImportError):
     """
     Indicates that a dependency for table reading is not present.
@@ -166,6 +170,7 @@ class OptionalTableImportError(ImportError):
     an ImportError.
     """
 
+
 class ParameterError(NotImplementedError):
     """
     Indicates that a reader cannot handle a passed parameter.
@@ -175,13 +180,15 @@ class ParameterError(NotImplementedError):
     C engine cannot handle.
     """
 
+
 class FastOptionsError(NotImplementedError):
     """
     Indicates that one of the specified options for fast
     reading is invalid.
     """
 
-class NoType(object):
+
+class NoType:
     """
     Superclass for ``StrType`` and ``NumType`` classes.
 
@@ -229,7 +236,7 @@ class AllType(StrType, FloatType, IntType):
     """
 
 
-class Column(object):
+class Column:
     """Table column.
 
     The key attributes of a Column object are:
@@ -240,6 +247,7 @@ class Column(object):
     * **str_vals** : list of column values as strings
     * **data** : list of converted column values
     """
+
     def __init__(self, name):
         self.name = name
         self.type = NoType  # Generic type (Int, Float, Str etc)
@@ -248,11 +256,15 @@ class Column(object):
         self.fill_values = {}
 
 
-class BaseInputter(object):
+class BaseInputter:
     """
     Get the lines from the table input and return a list of lines.
 
     """
+
+    encoding = None
+    """Encoding used to read the file"""
+
     def get_lines(self, table):
         """
         Get the lines from the ``table`` input. The input table can be one of:
@@ -277,8 +289,9 @@ class BaseInputter(object):
         try:
             if (hasattr(table, 'read') or
                     ('\n' not in table + '' and '\r' not in table + '')):
-                with get_readable_fileobj(table) as file_obj:
-                    table = file_obj.read()
+                with get_readable_fileobj(table,
+                                          encoding=self.encoding) as fileobj:
+                    table = fileobj.read()
             lines = table.splitlines()
         except TypeError:
             try:
@@ -306,7 +319,7 @@ class BaseInputter(object):
         return lines
 
 
-class BaseSplitter(object):
+class BaseSplitter:
     """
     Base splitter that uses python's split method to do the work.
 
@@ -384,16 +397,14 @@ class DefaultSplitter(BaseSplitter):
     csv_writer = None
     csv_writer_out = StringIO()
 
-
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
         whitespace-delimited files to prevent spurious columns at the beginning or end.
         If splitting on whitespace then replace unquoted tabs with space first"""
-        if self.delimiter == '\s':
+        if self.delimiter == r'\s':
             line = _replace_tab_with_space(line, self.escapechar, self.quotechar)
         return line.strip()
 
-
     def __call__(self, lines):
         """Return an iterator over the table ``lines``, where each iterator output
         is a list of the split line values.
@@ -411,20 +422,13 @@ class DefaultSplitter(BaseSplitter):
         if self.process_line:
             lines = [self.process_line(x) for x in lines]
 
-        # In Python 2.x the inputs to csv cannot be unicode.  In Python 3 these
-        # lines do nothing.
-        escapechar = None if self.escapechar is None else str(self.escapechar)
-        quotechar = None if self.quotechar is None else str(self.quotechar)
-        delimiter = None if self.delimiter is None else str(self.delimiter)
-
-        if delimiter == '\s':
-            delimiter = ' '
+        delimiter = ' ' if self.delimiter == r'\s' else self.delimiter
 
         csv_reader = csv.reader(lines,
                                 delimiter=delimiter,
                                 doublequote=self.doublequote,
-                                escapechar=escapechar,
-                                quotechar=quotechar,
+                                escapechar=self.escapechar,
+                                quotechar=self.quotechar,
                                 quoting=self.quoting,
                                 skipinitialspace=self.skipinitialspace
                                 )
@@ -436,16 +440,13 @@ class DefaultSplitter(BaseSplitter):
 
     def join(self, vals):
 
-        # In Python 2.x the inputs to csv cannot be unicode
-        escapechar = None if self.escapechar is None else str(self.escapechar)
-        quotechar = None if self.quotechar is None else str(self.quotechar)
         delimiter = ' ' if self.delimiter is None else str(self.delimiter)
 
         if self.csv_writer is None:
             self.csv_writer = CsvWriter(delimiter=delimiter,
                                         doublequote=self.doublequote,
-                                        escapechar=escapechar,
-                                        quotechar=quotechar,
+                                        escapechar=self.escapechar,
+                                        quotechar=self.quotechar,
                                         quoting=self.quoting,
                                         lineterminator='')
         if self.process_val:
@@ -464,7 +465,7 @@ def _replace_tab_with_space(line, escapechar, quotechar):
         String containing tabs to be replaced with spaces.
     escapechar : str
         Character in ``line`` used to escape special characters.
-    quotechar: str
+    quotechar : str
         Character in ``line`` indicating the start/end of a substring.
 
     Returns
@@ -502,11 +503,11 @@ def _get_line_index(line_or_func, lines):
         return line_or_func
 
 
-class BaseHeader(object):
+class BaseHeader:
     """
     Base table header reader
     """
-    auto_format = 'col%d'
+    auto_format = 'col{}'
     """ format string for auto-generating column names """
     start_line = None
     """ None, int, or a function of ``lines`` that returns None or int """
@@ -564,7 +565,8 @@ class BaseHeader(object):
                 raise InconsistentTableError('No data lines found so cannot autogenerate '
                                              'column names')
             n_data_cols = len(first_data_vals)
-            self.names = [self.auto_format % i for i in range(1, n_data_cols + 1)]
+            self.names = [self.auto_format.format(i)
+                          for i in range(1, n_data_cols + 1)]
 
         else:
             for i, line in enumerate(self.process_lines(lines)):
@@ -612,7 +614,7 @@ class BaseHeader(object):
             type_map_key = self.get_type_map_key(col)
             return self.col_type_map[type_map_key.lower()]
         except KeyError:
-            raise ValueError('Unknown data type ""%s"" for column "%s"' % (
+            raise ValueError('Unknown data type ""{}"" for column "{}"'.format(
                 col.raw_type, col.name))
 
     def check_column_names(self, names, strict_names, guessing):
@@ -645,14 +647,15 @@ class BaseHeader(object):
                                      .format(name))
         # When guessing require at least two columns
         if guessing and len(self.colnames) <= 1:
-            raise ValueError('Strict name guessing requires at least two columns')
+            raise ValueError('Table format guessing requires at least two columns, got {}'
+                             .format(list(self.colnames)))
 
         if names is not None and len(names) != len(self.colnames):
             raise ValueError('Length of names argument ({0}) does not match number'
                              ' of table columns ({1})'.format(len(names), len(self.colnames)))
 
 
-class BaseData(object):
+class BaseData:
     """
     Base table data reader.
     """
@@ -667,8 +670,7 @@ class BaseData(object):
     write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
     fill_include_names = None
     fill_exclude_names = None
-    # Currently, the default matches the numpy default for masked values.
-    fill_values = [(masked, '--')]
+    fill_values = [(masked, '')]
     formats = {}
 
     def __init__(self):
@@ -742,11 +744,11 @@ class BaseData(object):
             # when we write tables the columns may be astropy.table.Columns
             # which don't carry a fill_values by default
             for col in cols:
-                if ~hasattr(col, 'fill_values'):
+                if not hasattr(col, 'fill_values'):
                     col.fill_values = {}
 
             # if input is only one <fill_spec>, then make it a list
-            with ignored(TypeError):
+            with suppress(TypeError):
                 self.fill_values[0] + ''
                 self.fill_values = [self.fill_values]
 
@@ -834,7 +836,7 @@ def convert_numpy(numpy_type):
     ----------
     numpy_type : numpy data-type
         The numpy type required of an array returned by ``converter``. Must be a
-        valid `numpy type <http://docs.scipy.org/doc/numpy/user/basics.types.html>`_,
+        valid `numpy type <https://docs.scipy.org/doc/numpy/user/basics.types.html>`_,
         e.g. numpy.int, numpy.uint, numpy.int8, numpy.int64, numpy.float,
         numpy.float64, numpy.str.
 
@@ -894,7 +896,7 @@ def convert_numpy(numpy_type):
     return converter, converter_type
 
 
-class BaseOutputter(object):
+class BaseOutputter:
     """Output table as a dict of column objects keyed on column name.  The
     table data are stored as plain python lists within the column objects.
     """
@@ -916,8 +918,8 @@ class BaseOutputter(object):
                     converters_out.append((converter_func, converter_type))
 
         except (ValueError, TypeError):
-            raise ValueError('Error: invalid format for converters, see documentation\n%s' %
-                             converters)
+            raise ValueError('Error: invalid format for converters, see '
+                             'documentation\n{}'.format(converters))
         return converters_out
 
     def _convert_vals(self, cols):
@@ -969,6 +971,8 @@ class TableOutputter(BaseOutputter):
                           convert_numpy(numpy.str)]
 
     def __call__(self, cols, meta):
+        # Sets col.data to numpy array and col.type to io.ascii Type class (e.g.
+        # FloatType) for each col.
         self._convert_vals(cols)
 
         # If there are any values that were filled and tagged with a mask bit then this
@@ -991,7 +995,7 @@ class TableOutputter(BaseOutputter):
 
 class MetaBaseReader(type):
     def __init__(cls, name, bases, dct):
-        super(MetaBaseReader, cls).__init__(name, bases, dct)
+        super().__init__(name, bases, dct)
 
         format = dct.get('_format_name')
         if format is None:
@@ -1019,11 +1023,12 @@ class MetaBaseReader(type):
 
 
 def _is_number(x):
-    with ignored(ValueError):
+    with suppress(ValueError):
         x = float(x)
         return True
     return False
 
+
 def _apply_include_exclude_names(table, names, include_names, exclude_names):
     """
     Apply names, include_names and exclude_names to a table.
@@ -1036,7 +1041,7 @@ def _apply_include_exclude_names(table, names, include_names, exclude_names):
         List of names to override those in table (set to None to use existing names)
     include_names : list
         List of names to include in output
-    exclude_names: list
+    exclude_names : list
         List of names to exclude from output (applied after ``include_names``)
 
     """
@@ -1062,8 +1067,7 @@ def _apply_include_exclude_names(table, names, include_names, exclude_names):
         table.remove_columns(remove_names)
 
 
-@six.add_metaclass(MetaBaseReader)
-class BaseReader(object):
+class BaseReader(metaclass=MetaBaseReader):
     """Class providing methods to read and write an ASCII table using the specified
     header, data, inputter, and outputter instances.
 
@@ -1082,6 +1086,7 @@ class BaseReader(object):
     exclude_names = None
     strict_names = False
     guessing = False
+    encoding = None
 
     header_class = BaseHeader
     data_class = BaseData
@@ -1132,7 +1137,7 @@ class BaseReader(object):
         # If ``table`` is a file then store the name in the ``data``
         # attribute. The ``table`` is a "file" if it is a string
         # without the new line specific to the OS.
-        with ignored(TypeError):
+        with suppress(TypeError):
             # Strings only
             if os.linesep not in table + '':
                 self.data.table_name = os.path.basename(table)
@@ -1166,20 +1171,22 @@ class BaseReader(object):
 
                 # otherwise, we raise an error only if it is still inconsistent
                 if len(str_vals) != n_cols:
-                    errmsg = ('Number of header columns (%d) inconsistent with '
-                              'data columns (%d) at data line %d\n'
-                              'Header values: %s\n'
-                              'Data values: %s' % (n_cols, len(str_vals), i,
-                                                   [x.name for x in cols], str_vals))
+                    errmsg = ('Number of header columns ({}) inconsistent with'
+                              ' data columns ({}) at data line {}\n'
+                              'Header values: {}\n'
+                              'Data values: {}'.format(
+                            n_cols, len(str_vals), i,
+                            [x.name for x in cols], str_vals))
+
                     raise InconsistentTableError(errmsg)
 
             for j, col in enumerate(cols):
                 col.str_vals.append(str_vals[j])
 
         self.data.masks(cols)
-        table = self.outputter(cols, self.meta)
         if hasattr(self.header, 'table_meta'):
-            table.meta.update(self.header.table_meta)
+            self.meta['table'].update(self.header.table_meta)
+        table = self.outputter(cols, self.meta)
         self.cols = self.header.cols
 
         _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
@@ -1226,6 +1233,26 @@ class BaseReader(object):
             comment_lines = []
         return comment_lines
 
+    def update_table_data(self, table):
+        """
+        Update table columns in place if needed.
+
+        This is a hook to allow updating the table columns after name
+        filtering but before setting up to write the data.  This is currently
+        only used by ECSV and is otherwise just a pass-through.
+
+        Parameters
+        ----------
+        table : `astropy.table.Table`
+            Input table for writing
+
+        Returns
+        -------
+        table : `astropy.table.Table`
+            Output table for writing
+        """
+        return table
+
     def write_header(self, lines, meta):
         self.header.write_comments(lines, meta)
         self.header.write(lines)
@@ -1247,13 +1274,21 @@ class BaseReader(object):
         """
 
         # Check column names before altering
-        self.header.cols = list(six.itervalues(table.columns))
+        self.header.cols = list(table.columns.values())
         self.header.check_column_names(self.names, self.strict_names, False)
 
+        # In-place update of columns in input ``table`` to reflect column
+        # filtering.  Note that ``table`` is guaranteed to be a copy of the
+        # original user-supplied table.
         _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
 
+        # This is a hook to allow updating the table columns after name
+        # filtering but before setting up to write the data.  This is currently
+        # only used by ECSV and is otherwise just a pass-through.
+        table = self.update_table_data(table)
+
         # Now use altered columns
-        new_cols = list(six.itervalues(table.columns))
+        new_cols = list(table.columns.values())
         # link information about the columns to the writer object (i.e. self)
         self.header.cols = new_cols
         self.data.cols = new_cols
@@ -1320,9 +1355,10 @@ class WhitespaceSplitter(DefaultSplitter):
 
         return ''.join(newline)
 
+
 extra_reader_pars = ('Reader', 'Inputter', 'Outputter',
                      'delimiter', 'comment', 'quotechar', 'header_start',
-                     'data_start', 'data_end', 'converters',
+                     'data_start', 'data_end', 'converters', 'encoding',
                      'data_Splitter', 'header_Splitter',
                      'names', 'include_names', 'exclude_names', 'strict_names',
                      'fill_values', 'fill_include_names', 'fill_exclude_names')
@@ -1335,13 +1371,13 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
     """
 
     from .fastbasic import FastBasic
-    if issubclass(Reader, FastBasic): # Fast readers handle args separately
+    if issubclass(Reader, FastBasic):  # Fast readers handle args separately
         if Inputter is not None:
             kwargs['Inputter'] = Inputter
         return Reader(**kwargs)
 
     if 'fast_reader' in kwargs:
-        del kwargs['fast_reader'] # ignore fast_reader parameter for slow readers
+        del kwargs['fast_reader']  # ignore fast_reader parameter for slow readers
     reader_kwargs = dict([k, v] for k, v in kwargs.items() if k not in extra_reader_pars)
     reader = Reader(**reader_kwargs)
 
@@ -1407,9 +1443,13 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
         reader.data.fill_include_names = kwargs['fill_include_names']
     if 'fill_exclude_names' in kwargs:
         reader.data.fill_exclude_names = kwargs['fill_exclude_names']
+    if 'encoding' in kwargs:
+        reader.encoding = kwargs['encoding']
+        reader.inputter.encoding = kwargs['encoding']
 
     return reader
 
+
 extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
                      'strip_whitespace',
                      'names', 'include_names', 'exclude_names',
@@ -1424,7 +1464,14 @@ def _get_writer(Writer, fast_writer, **kwargs):
 
     from .fastbasic import FastBasic
 
-    if issubclass(Writer, FastBasic): # Fast writers handle args separately
+    # A value of None for fill_values imply getting the default string
+    # representation of masked values (depending on the writer class), but the
+    # machinery expects a list.  The easiest here is to just pop the value off,
+    # i.e. fill_values=None is the same as not providing it at all.
+    if 'fill_values' in kwargs and kwargs['fill_values'] is None:
+        del kwargs['fill_values']
+
+    if issubclass(Writer, FastBasic):  # Fast writers handle args separately
         return Writer(**kwargs)
     elif fast_writer and 'fast_{0}'.format(Writer._format_name) in FAST_CLASSES:
         # Switch to fast writer
@@ -1450,7 +1497,7 @@ def _get_writer(Writer, fast_writer, **kwargs):
             # Restore the default SplitterClass process_val method which strips
             # whitespace.  This may have been changed in the Writer
             # initialization (e.g. Rdb and Tab)
-            writer.data.splitter.process_val = lambda x: x.strip()
+            writer.data.splitter.process_val = operator.methodcaller('strip')
         else:
             writer.data.splitter.process_val = None
     if 'names' in kwargs:
@@ -1460,7 +1507,13 @@ def _get_writer(Writer, fast_writer, **kwargs):
     if 'exclude_names' in kwargs:
         writer.exclude_names = kwargs['exclude_names']
     if 'fill_values' in kwargs:
-        writer.data.fill_values = kwargs['fill_values']
+        # Prepend user-specified values to the class default.
+        with suppress(TypeError, IndexError):
+            # Test if it looks like (match, replace_string, optional_colname),
+            # in which case make it a list
+            kwargs['fill_values'][1] + ''
+            kwargs['fill_values'] = [kwargs['fill_values']]
+        writer.data.fill_values = kwargs['fill_values'] + writer.data.fill_values
     if 'fill_include_names' in kwargs:
         writer.data.fill_include_names = kwargs['fill_include_names']
     if 'fill_exclude_names' in kwargs:
