@@ -8,17 +8,18 @@ html.py:
 must be installed to read HTML tables.
 """
 
-from __future__ import absolute_import, division, print_function
-from ...extern import six
-from ...extern.six.moves import zip as izip
+
+import warnings
+import numpy
+
 
 from . import core
 from ...table import Column
-from ...table.column import col_iter_str_vals, col_getattr
 from ...utils.xml import writer
 
 from copy import deepcopy
 
+
 class SoupString(str):
     """
     Allows for strings to hold BeautifulSoup data.
@@ -30,6 +31,7 @@ class SoupString(str):
     def __init__(self, val):
         self.soup = val
 
+
 class ListWriter:
     """
     Allows for XMLWriter to write to a list instead of a file.
@@ -41,6 +43,7 @@ class ListWriter:
     def write(self, data):
         self.out.append(data)
 
+
 def identify_table(soup, htmldict, numtable):
     """
     Checks whether the given BeautifulSoup tag is the table
@@ -48,13 +51,13 @@ def identify_table(soup, htmldict, numtable):
     """
 
     if soup is None or soup.name != 'table':
-        return False # Tag is not a <table>
+        return False  # Tag is not a <table>
 
     elif 'table_id' not in htmldict:
         return numtable == 1
     table_id = htmldict['table_id']
 
-    if isinstance(table_id, six.string_types):
+    if isinstance(table_id, str):
         return 'id' in soup.attrs and soup['id'] == table_id
     elif isinstance(table_id, int):
         return table_id == numtable
@@ -84,13 +87,16 @@ class HTMLInputter(core.BaseInputter):
                                         'installed to read HTML tables')
 
         if 'parser' not in self.html:
-            soup = BeautifulSoup('\n'.join(lines))
-        else: # use a custom backend parser
+            with warnings.catch_warnings():
+                # Ignore bs4 parser warning #4550.
+                warnings.filterwarnings('ignore', '.*no parser was explicitly specified.*')
+                soup = BeautifulSoup('\n'.join(lines))
+        else:  # use a custom backend parser
             soup = BeautifulSoup('\n'.join(lines), self.html['parser'])
         tables = soup.find_all('table')
         for i, possible_table in enumerate(tables):
             if identify_table(possible_table, self.html, i + 1):
-                table = possible_table # Find the correct table
+                table = possible_table  # Find the correct table
                 break
         else:
             if isinstance(self.html['table_id'], int):
@@ -105,6 +111,7 @@ class HTMLInputter(core.BaseInputter):
 
         return soup_list
 
+
 class HTMLSplitter(core.BaseSplitter):
     """
     Split HTML table data.
@@ -130,6 +137,7 @@ class HTMLSplitter(core.BaseSplitter):
             raise core.InconsistentTableError('HTML tables must contain data '
                                               'in a <table> tag')
 
+
 class HTMLOutputter(core.TableOutputter):
     """
     Output the HTML data as an ``astropy.table.Table`` object.
@@ -139,6 +147,11 @@ class HTMLOutputter(core.TableOutputter):
     of <th>).
     """
 
+    default_converters = [core.convert_numpy(numpy.int),
+                          core.convert_numpy(numpy.float),
+                          core.convert_numpy(numpy.str),
+                          core.convert_numpy(numpy.unicode)]
+
     def __call__(self, cols, meta):
         """
         Process the data in multidimensional columns.
@@ -152,14 +165,14 @@ class HTMLOutputter(core.TableOutputter):
                 # Join elements of spanned columns together into list of tuples
                 span_cols = cols[col_num:col_num + col.colspan]
                 new_col = core.Column(col.name)
-                new_col.str_vals = list(izip(*[x.str_vals for x in span_cols]))
+                new_col.str_vals = list(zip(*[x.str_vals for x in span_cols]))
                 new_cols.append(new_col)
                 col_num += col.colspan
             else:
                 new_cols.append(col)
                 col_num += 1
 
-        return super(HTMLOutputter, self).__call__(new_cols, meta)
+        return super().__call__(new_cols, meta)
 
 
 class HTMLHeader(core.BaseHeader):
@@ -241,9 +254,9 @@ class HTMLData(core.BaseData):
             return None
         return last_index + 1
 
+
 class HTML(core.BaseReader):
-    """
-    Read and write HTML tables.
+    """Read and write HTML tables.
 
     In order to customize input and output, a dict of parameters may
     be passed to this class holding specific customizations.
@@ -266,6 +279,20 @@ class HTML(core.BaseReader):
             use the syntax 1.36583e-13 .. 1.36583e-13 for output. If not
             present, this parameter will be true by default.
 
+        * raw_html_cols : column name or list of names with raw HTML content
+            This allows one to include raw HTML content in the column output,
+            for instance to include link references in a table.  This option
+            requires that the bleach package be installed.  Only whitelisted
+            tags are allowed through for security reasons (see the
+            raw_html_clean_kwargs arg).
+
+        * raw_html_clean_kwargs : dict of keyword args controlling HTML cleaning
+            Raw HTML will be cleaned to prevent unsafe HTML from ending up in
+            the table output.  This is done by calling ``bleach.clean(data,
+            **raw_html_clean_kwargs)``.  For details on the available options
+            (e.g. tag whitelist) see:
+            http://bleach.readthedocs.io/en/latest/clean.html
+
         * parser : Specific HTML parsing library to use
             If specified, this specifies which HTML parsing library
             BeautifulSoup should use as a backend. The options to choose
@@ -280,6 +307,9 @@ class HTML(core.BaseReader):
         * cssfiles : list of css files to include when writing table.
 
         * js : js script to include in the body when writing table.
+
+        * table_class : css class for the table
+
     """
 
     _format_name = 'html'
@@ -295,7 +325,7 @@ class HTML(core.BaseReader):
         """
         Initialize classes for HTML reading and writing.
         """
-        super(HTML, self).__init__()
+        super().__init__()
         self.html = deepcopy(htmldict)
         if 'multicol' not in htmldict:
             self.html['multicol'] = True
@@ -309,26 +339,42 @@ class HTML(core.BaseReader):
         """
 
         self.outputter = HTMLOutputter()
-        return core.BaseReader.read(self, table)
+        return super().read(table)
 
     def write(self, table):
         """
         Return data in ``table`` converted to HTML as a list of strings.
         """
+        cols = list(table.columns.values())
+
+        self.data.header.cols = cols
+
+        if isinstance(self.data.fill_values, tuple):
+            self.data.fill_values = [self.data.fill_values]
+
+        self.data._set_fill_values(cols)
 
-        cols = list(six.itervalues(table.columns))
         lines = []
 
+        # Set HTML escaping to False for any column in the raw_html_cols input
+        raw_html_cols = self.html.get('raw_html_cols', [])
+        if isinstance(raw_html_cols, str):
+            raw_html_cols = [raw_html_cols]  # Allow for a single string as input
+        cols_escaped = [col.info.name not in raw_html_cols for col in cols]
+
+        # Kwargs that get passed on to bleach.clean() if that is available.
+        raw_html_clean_kwargs = self.html.get('raw_html_clean_kwargs', {})
+
         # Use XMLWriter to output HTML to lines
         w = writer.XMLWriter(ListWriter(lines))
 
         with w.tag('html'):
             with w.tag('head'):
                 # Declare encoding and set CSS style for table
-                with w.tag('meta', attrib={'charset':'utf-8'}):
+                with w.tag('meta', attrib={'charset': 'utf-8'}):
                     pass
-                with w.tag('meta', attrib={'http-equiv':'Content-type',
-                                    'content':'text/html;charset=UTF-8'}):
+                with w.tag('meta', attrib={'http-equiv': 'Content-type',
+                                    'content': 'text/html;charset=UTF-8'}):
                     pass
                 if 'css' in self.html:
                     with w.tag('style'):
@@ -343,13 +389,19 @@ class HTML(core.BaseReader):
                             w.data('')  # need this instead of pass to get <script></script>
             with w.tag('body'):
                 if 'js' in self.html:
-                    with w.tag('script'):
-                        w.data(self.html['js'])
-                if isinstance(self.html['table_id'], six.string_types):
+                    with w.xml_cleaning_method('none'):
+                        with w.tag('script'):
+                            w.data(self.html['js'])
+                if isinstance(self.html['table_id'], str):
                     html_table_id = self.html['table_id']
                 else:
                     html_table_id = None
-                with w.tag('table', id=html_table_id):
+                if 'table_class' in self.html:
+                    html_table_class = self.html['table_class']
+                    attrib = {"class": html_table_class}
+                else:
+                    attrib = {}
+                with w.tag('table', id=html_table_id, attrib=attrib):
                     with w.tag('thead'):
                         with w.tag('tr'):
                             for col in cols:
@@ -358,25 +410,57 @@ class HTML(core.BaseReader):
                                     w.start('th', colspan=col.shape[1])
                                 else:
                                     w.start('th')
-                                w.data(col_getattr(col, 'name').strip())
+                                w.data(col.info.name.strip())
                                 w.end(indent=False)
                         col_str_iters = []
-                        for col in cols:
+                        new_cols_escaped = []
+                        for col, col_escaped in zip(cols, cols_escaped):
                             if len(col.shape) > 1 and self.html['multicol']:
                                 span = col.shape[1]
                                 for i in range(span):
                                     # Split up multicolumns into separate columns
                                     new_col = Column([el[i] for el in col])
-                                    col_str_iters.append(col_iter_str_vals(new_col))
-                            else:
-                                col_str_iters.append(col_iter_str_vals(col))
 
-                    for row in izip(*col_str_iters):
+                                    new_col_iter_str_vals = self.fill_values(col, new_col.info.iter_str_vals())
+                                    col_str_iters.append(new_col_iter_str_vals)
+                                    new_cols_escaped.append(col_escaped)
+                            else:
+
+                                col_iter_str_vals = self.fill_values(col, col.info.iter_str_vals())
+                                col_str_iters.append(col_iter_str_vals)
+
+                                new_cols_escaped.append(col_escaped)
+
+                    for row in zip(*col_str_iters):
                         with w.tag('tr'):
-                            for el in row:
-                                w.start('td')
-                                w.data(el.strip())
-                                w.end(indent=False)
+                            for el, col_escaped in zip(row, new_cols_escaped):
+                                # Potentially disable HTML escaping for column
+                                method = ('escape_xml' if col_escaped else 'bleach_clean')
+                                with w.xml_cleaning_method(method, **raw_html_clean_kwargs):
+                                    w.start('td')
+                                    w.data(el.strip())
+                                    w.end(indent=False)
 
         # Fixes XMLWriter's insertion of unwanted line breaks
         return [''.join(lines)]
+
+    def fill_values(self, col, col_str_iters):
+        """
+        Return an iterator of the values with replacements based on fill_values
+        """
+        # check if the col is a masked column and has fill values
+        is_masked_column = hasattr(col, 'mask')
+        has_fill_values = hasattr(col, 'fill_values')
+
+        for idx, col_str in enumerate(col_str_iters):
+            if is_masked_column and has_fill_values:
+                if col.mask[idx]:
+                    yield col.fill_values[core.masked]
+                    continue
+
+            if has_fill_values:
+                if col_str in col.fill_values:
+                    yield col.fill_values[col_str]
+                    continue
+
+            yield col_str
