@@ -7,7 +7,6 @@ Built on daophot.py:
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
 
-from __future__ import absolute_import, division, print_function
 
 import re
 
@@ -55,7 +54,7 @@ class SExtractorHeader(core.BaseHeader):
                     colnumber = int(match.group('colnumber'))
                     colname = match.group('colname')
                     coldescr = match.group('coldescr')
-                    colunit = match.group('colunit') # If no units are given, colunit = None
+                    colunit = match.group('colunit')  # If no units are given, colunit = None
                     columns[colnumber] = (colname, coldescr, colunit)
         # Handle skipped column numbers
         colnumbers = sorted(columns)
@@ -75,7 +74,7 @@ class SExtractorHeader(core.BaseHeader):
             for n in colnumbers:
                 if n != previous_column + 1:
                     for c in range(previous_column+1, n):
-                        column_name = columns[previous_column][0]+"_%d" % (c-previous_column)
+                        column_name = columns[previous_column][0]+"_{}".format(c-previous_column)
                         column_descr = columns[previous_column][1]
                         column_unit = columns[previous_column][2]
                         columns[c] = (column_name, column_descr, column_unit)
@@ -140,7 +139,7 @@ class SExtractor(core.BaseReader):
         Read input data (file-like object, filename, list of strings, or
         single string) into a Table and return the result.
         """
-        out = super(SExtractor, self).read(table)
+        out = super().read(table)
         # remove the comments
         if 'comments' in out.meta:
             del out.meta['comments']
