@@ -10,9 +10,10 @@
 #include <math.h>
 #include <float.h>
 #include <ctype.h>
+#include <sys/types.h>
 
 #ifdef _MSC_VER
-#define inline __inline
+    #define inline __inline
     #ifndef NAN
         static const unsigned long __nan[2] = {0xffffffff, 0x7fffffff};
         #define NAN (*(const double *) __nan)
@@ -21,6 +22,13 @@
         static const unsigned long __infinity[2] = {0x00000000, 0x7ff00000};
         #define INFINITY (*(const double *) __infinity)
     #endif
+#else
+    #ifndef INFINITY
+        #define INFINITY (1.0/0.0)
+    #endif
+    #ifndef NAN
+        #define NAN (INFINITY-INFINITY)
+    #endif
 #endif
 
 typedef enum
@@ -32,7 +40,6 @@ typedef enum
     QUOTED_FIELD,
     QUOTED_FIELD_NEWLINE,
     COMMENT,
-    CARRIAGE_RETURN
 } tokenizer_state;
 
 typedef enum
@@ -48,11 +55,12 @@ typedef enum
 typedef struct
 {
     char *source;          // single string containing all of the input
-    int source_len;        // length of the input
-    int source_pos;        // current index in source for tokenization
+    size_t source_len;      // length of the input
+    size_t source_pos;      // current index in source for tokenization
     char delimiter;        // delimiter character
     char comment;          // comment character
     char quotechar;        // quote character
+    char expchar;          // exponential character in scientific notation
     char **output_cols;    // array of output strings for each column
     char **col_ptrs;       // array of pointers to current output position for each col
     int *output_len;       // length of each output column string
@@ -83,9 +91,9 @@ output_cols: ["A\x0010\x001", "B\x005.\x002", "C\x006\x003"]
 #define INITIAL_COL_SIZE 500
 #define INITIAL_COMMENT_LEN 50
 
-tokenizer_t *create_tokenizer(char delimiter, char comment, char quotechar, int fill_extra_cols,
-                              int strip_whitespace_lines, int strip_whitespace_fields,
-                              int use_fast_converter);
+tokenizer_t *create_tokenizer(char delimiter, char comment, char quotechar, char expchar,
+                              int fill_extra_cols, int strip_whitespace_lines,
+                              int strip_whitespace_fields, int use_fast_converter);
 void delete_tokenizer(tokenizer_t *tokenizer);
 void delete_data(tokenizer_t *tokenizer);
 void resize_col(tokenizer_t *self, int index);
@@ -95,11 +103,11 @@ int tokenize(tokenizer_t *self, int end, int header, int num_cols);
 long str_to_long(tokenizer_t *self, char *str);
 double str_to_double(tokenizer_t *self, char *str);
 double xstrtod(const char *str, char **endptr, char decimal,
-               char sci, char tsep, int skip_trailing);
+               char expchar, char tsep, int skip_trailing);
 void start_iteration(tokenizer_t *self, int col);
 char *next_field(tokenizer_t *self, int *size);
 long file_len(FILE *fhandle);
-char *get_line(char *ptr, int *len, int map_len);
+char *get_line(char *ptr, size_t *len, size_t map_len);
 void reset_comments(tokenizer_t *self);
 
 #endif
